#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
=========================
Module: pergola.parsers
=========================

.. module:: parsers

This module provides the way to read scripts options provided by pergola library.


"""
from __future__ import print_function
from __future__ import absolute_import

from scipy.io  import loadmat
from os.path   import join, exists
from .mapping   import MappingInfo, check_path
from tempfile  import NamedTemporaryFile

from .intervals import IntData
from numpy     import hstack, ndenumerate, mean, divide 
from shutil    import copyfileobj

_csv_file_ext = ".csv"

###############
### JAABA stuff

def jaaba_scores_to_csv(input_file, name_file="JAABA_scores", mode="w", delimiter="\t", path_w=None, norm=False, data_type="a"):
    """   
    Creates a csv file from a scores file produced using JAABA and in matlab format
        
    :param mode: :py:func:`str` mode to use by default write
    :param "\t" delimiter: :py:func:`str` Character use to separate values of 
        the same record in file (default "\t").
    :param None path_w: :py:func:`str` path to dump the files, by default None
    :param False norm: set whether data should be normalize (-1,1) using normalization
        factor contained in the file
    :param data_type: :py:func:`str` data type in the file "behavior" e.g. chase
    
    """
    
    path = ""
    header = ["animal", "startTime", "endTime", "value", "dataType"]
    if not path_w: 
        path = getcwd()
        print('CSV file will be dump into \"%s\" ' \
                       'as not path has been set in path_w' % (path), file=stderr)
    else:
        if exists(path_w):
            path = path_w
        else:
            raise IOError('Provided path does not exists: %s' % path_w)
    
    input_file = check_path(input_file)
    jaaba_data = loadmat(input_file)
    
    # Checking JAABA version
    version_jaaba = hstack(hstack(hstack(jaaba_data['version'])))[0][0]
    
    if version_jaaba != '0.5.1':
        print('WARNING: JAABA version is not 0.5.1 but %s, this might cause ' \
                        'problems if the structure of JAABA files has changed.' \
                        % (version_jaaba), file=stderr)
    
    # Structure of the file can be find here:
    # http://jaaba.sourceforge.net/ApplyingAClassifier.html#ScoresFile
    
    start_times = jaaba_data['allScores']['t0s']
    end_times = jaaba_data['allScores']['t1s']
    scores = jaaba_data['allScores']['scores']
    score_norm = jaaba_data['allScores']['scoreNorm']
    
    start_times_flat = hstack(hstack(hstack(start_times)))
    end_times_flat = hstack(hstack(hstack(end_times)))
    scores_flat = hstack(hstack(hstack(scores)))
    score_norm = hstack(hstack(score_norm))[0][0]
   
    scoreFile = open(join(path, name_file + _csv_file_ext), mode)
    scoreFile.write(delimiter.join(header) + "\n")
    
    if norm:
        # Dirty way of solving problem with ipython notebook, division was not working there
        score_norm = float(score_norm)
        scores_flat = divide(scores_flat, score_norm)
    
    for idx_animal, start_times_animal in enumerate (start_times_flat):
        start_times_animal= hstack(start_times_animal)
        end_times_animal = hstack(end_times_flat [idx_animal])
        scores_animal = hstack(scores_flat [idx_animal]) 
    
        for idx_time, start_time in enumerate (start_times_animal):
            end_time = end_times_animal[idx_time]
            mean_score = mean(scores_animal[start_time:end_time])            
            # Because we use the convention that the animal is performing the behavior 
            # from frame t to t+1 if it is labeled/classified as performing the behavior 
            # at frame t, allScores.postprocessed{i}(allScores.t1s{i}(j)) will be 0 and 
            # allScores.postprocessed{i}(allScores.t0s{i}(j)) will be 1.
            # that is why I substract one to the end_time
            # In fact in the graphical interface it starts at start_time - 0.5 and ends in 
            # end_time - 0.5
            scoreFile.write(delimiter.join('{}'.format(v) for v in [idx_animal+1, start_time, end_time -1, mean_score, data_type]) + "\n") 

    scoreFile.close()

def jaaba_scores_to_intData(input_file, map_jaaba, name_file="JAABA_scores", delimiter="\t", norm=False, data_type="a"):
    """   
    Creates a csv file from a scores file produced using JAABA and in matlab format
    
    :param input_file: path to the JAABA file in matlab format
    :param map_jaaba: path to the mapping files between JAABA data and pergola ontology 
    :param "\t" delimiter: :py:func:`str` Character use to separate values of 
        the same record in file (default "\t").
    :param False norm: set whether data should be normalize (-1,1) using normalization
        factor contained in the file
    :param data_type: :py:func:`str` data type in the file "behavior" e.g. chase
    
    :returns: IntData object
    
    """

    path = ""
    header = ["animal", "startTime", "endTime", "value", "dataType"]
    input_file = check_path(input_file)
    jaaba_data = loadmat(input_file)
        
    # Checking JAABA version
    version_jaaba = hstack(hstack(hstack(jaaba_data['version'])))[0][0]
    
    if version_jaaba != '0.5.1':
        print('WARNING: JAABA version is not 0.5.1 but %s, this might cause ' \
                        'problems if the structure of JAABA files has changed.' \
                        % (version_jaaba), file=stderr)
    
    # Structure of the file can be find here:
    # http://jaaba.sourceforge.net/ApplyingAClassifier.html#ScoresFile
    start_times = jaaba_data['allScores']['t0s']
    end_times = jaaba_data['allScores']['t1s']
    scores = jaaba_data['allScores']['scores']
    score_norm = jaaba_data['allScores']['scoreNorm']
    
    start_times_flat = hstack(hstack(hstack(start_times)))
    end_times_flat = hstack(hstack(hstack(end_times)))
    scores_flat = hstack(hstack(hstack(scores)))
    score_norm = hstack(hstack(score_norm))[0][0]
    
    temp = NamedTemporaryFile(delete=True, mode='w')
    temp.write(delimiter.join(header) + "\n")
    
    if norm:
        # Dirty way of solving problem with ipython notebook, division was not working there
        score_norm = float(score_norm)
        scores_flat = divide(scores_flat, score_norm)
         
    for idx_animal, start_times_animal in enumerate (start_times_flat):
        start_times_animal= hstack(start_times_animal)
        end_times_animal = hstack(end_times_flat [idx_animal])
        scores_animal = hstack(scores_flat [idx_animal]) 
    
        for idx_time, start_time in enumerate (start_times_animal):
            end_time = end_times_animal[idx_time]
            mean_score = mean(scores_animal[start_time:end_time])            
            # Because we use the convention that the animal is performing the behavior 
            # from frame t to t+1 if it is labeled/classified as performing the behavior 
            # at frame t, allScores.postprocessed{i}(allScores.t1s{i}(j)) will be 0 and 
            # allScores.postprocessed{i}(allScores.t0s{i}(j)) will be 1.
            # that is why I substract one to the end_time
            # In fact in the graphical interface it starts at start_time - 0.5 and ends in 
            # end_time - 0.5

            temp.write(delimiter.join('{}'.format(v) for v in [idx_animal+1, start_time, end_time -1, mean_score, data_type]) + "\n")

    # rewinds the file handle
    temp.seek(0)

    map_jaaba = check_path(map_jaaba)
    map = MappingInfo(map_jaaba)
    
    int_data_jaaba = IntData(temp.name, map_dict = map.correspondence)     
    temp.close()
    
    return (int_data_jaaba)

def extract_jaaba_features(dir_perframe,  output="csv", map_jaaba=False, delimiter="\t", feature="velmag", path_w=""):
    """   
    Creates a csv file or a IntData object from feature mat files dumped by JAABA in perframe directory in matlab format
    
    :param dir_perframe: path to the JAABA directory where perframe features are dumped
    :param "csv" output: :py:func:`str` sets whether data has to be extracted to a csv or an IntData object
    :param map_jaaba: path to the mapping files between JAABA data and pergola ontology 
    :param "\t" delimiter: :py:func:`str` Character used in the csv output file to separate values of 
        the same record (default "\t").
    :param "velmag" feature: :py:func:`str` data type (feature) to extract e.g. velmag (speed of the center of rotation)
        More features can be found in http://ctrax.sourceforge.net/bmat.html
    :param None path_w: :py:func:`str` path to dump the files
    
    returns: IntData object
    
    """
    
    input_path = join(dir_perframe, feature + ".mat")
        
    input_file = check_path(input_path)
        
    jaaba_feature = loadmat(input_file)
    
    output_option = ["csv", "IntData"]
    
    if output not in output_option:
            raise ValueError("Option output \'%s\' not allowed. Possible values are %s"%(output_options, ', '.join(['{}'.format(m) for m in output_options])))
    
    temp = NamedTemporaryFile()
    header = ["animal", "startTime", "endTime", "value", "dataType"]
    temp.write(delimiter.join(header) + "\n")
    
    for id_animal, animal_jaaba_feature in enumerate (jaaba_feature['data'][0]):
        animal_jaaba_feature= hstack(animal_jaaba_feature)
        
        for t, v in ndenumerate(animal_jaaba_feature):             
            temp.write(delimiter.join('{}'.format(v) for v in [id_animal+1, t[0], t[0]+1, v, feature]) + "\n")
    
    # rewinds the file handle
    temp.seek(0)
    
    if output == "csv":
        if not path_w: 
            path = getcwd()
            print('CSV file will be dump into \"%s\" ' \
                           'as not path has been set in path_w' % (path), file=stderr)
        else:
            if exists(path_w):
                path = path_w
            else:
                raise IOError('Provided path does not exists: %s' % path_w)
                
        feature_file = open(join(path, feature + _csv_file_ext), "wb")
                
        copyfileobj(temp, feature_file)
        temp.close()
        
    elif output == "IntData":                        
        map_jaaba = check_path(map_jaaba)
        map = MappingInfo(map_jaaba)
        
        int_data_jaaba = IntData(temp.name, map_dict = map.correspondence)     
        temp.close()
        
        return (int_data_jaaba)



#!/usr/bin/env python
#
#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function
from __future__ import absolute_import
from pergola import parsers
from pergola import jaaba_parsers
from argparse import ArgumentParser
from sys      import stderr, exit
from . import pergola_rules
from os.path import dirname, abspath, basename, splitext
from os import chdir, getcwd
from shutil import copy, rmtree

from tempfile import NamedTemporaryFile, mkdtemp

def main(args=None):
    """
    main function
    """
    parser_jaaba_parser = ArgumentParser(parents=[parsers.jaaba_parser])        
    
    args = parser_jaaba_parser.parse_args()
    
#     jaaba_to_pergola(option=args.command)
    jaaba_to_pergola(option=args.command, args=args)
    
def jaaba_to_pergola(option, args):
    """
    main function
    sc: scores to csv 
    sp: scores to pergola
    fc: features to csv
    fp: features to pergola
    """

    for input_file in args.input:
        if option == "sc" or option =="sp":

            print("@@@jaaba_to_pergola.py: Input file is %s" % input_file, file=stderr)

            tmp_track = NamedTemporaryFile(prefix='jaaba_csv', suffix='.csv', delete=True)

            name_tmp = splitext(basename(tmp_track.name))[0]
            path_out = dirname(abspath(tmp_track.name))

            jaaba_parsers.jaaba_scores_to_csv(input_file=input_file, path_w=path_out, name_file=name_tmp, norm=True, data_type="chase")

            output_file = tmp_track.name

            if option == "sc":
                path_w = dirname(abspath(input_file))
                f_out = path_w + '/' + 'JAABA_scores.csv'

                copy (tmp_track.name, f_out)
                print("@@@jaaba_to_pergola.py: Scores dumped in %s" % f_out, file=stderr)

            elif option == "sp":
                pergola_rules.pergola_rules(path=output_file, map_file_path=args.mapping_file, sel_tracks=args.tracks,
                      list=args.list, range=args.range, track_actions=args.track_actions,
                      data_types_actions=args.data_types_actions, data_types_list=args.data_types_list,
                      write_format=args.format, relative_coord=args.relative_coord,
                      intervals_gen=args.intervals_gen, multiply_f=args.multiply_intervals,
                      no_header=args.no_header, fields2read=args.fields_read, window_size=args.window_size,
                      no_track_line=args.no_track_line, separator=args.field_separator,
                      bed_lab_sw=args.bed_label, color_dict=args.color_file, window_mean=args.window_mean)

        elif option == "fc" or option == "fp":
            print("@@@jaaba_to_pergola.py: Extracting Jaaba features from %s" % input_file, file=stderr)
            print("@@@jaaba_to_pergola.py: Selected feature/s %s" %args.feature, file=stderr)


            if not args.dumping_dir:
                dumping_dir = getcwd()
                print("@@@jaaba_to_pergola.py: No path selected, files dump into path: ", dumping_dir, file=stderr)
            else:
                dumping_dir = args.dumping_dir
                print("@@@jaaba_to_pergola.py: Files dump into path: ", dumping_dir, file=stderr)

            path_tmp = mkdtemp()

            for f in args.feature:
                jaaba_parsers.extract_jaaba_features(dir_perframe=input_file, map_jaaba="", delimiter="\t",
                                               feature=f, output="csv", path_w=path_tmp)

                tmp_file = path_tmp + '/' + f + '.csv'

                if option == "fc":
                    f_out = dumping_dir + '/' + f +  '.csv'

                    copy (tmp_file, f_out)

                elif option == "fp":
                    chdir(dumping_dir)
    #                 pergola_rules.main(path=tmp_file, map_file_path=args.mapping_file, sel_tracks=args.tracks,
                    pergola_rules.pergola_rules(path=tmp_file, map_file_path=args.mapping_file, sel_tracks=args.tracks,
                      list=args.list, range=args.range, track_actions=args.track_actions,
                      data_types_actions=args.data_types_actions, data_types_list=args.data_types_list,
                      write_format=args.format, relative_coord=args.relative_coord,
                      intervals_gen=args.intervals_gen, multiply_f=args.multiply_intervals,
                      no_header=args.no_header, fields2read=args.fields_read, window_size=args.window_size,
                      no_track_line=args.no_track_line, separator=args.field_separator,
                      bed_lab_sw=args.bed_label, color_dict=args.color_file, window_mean=args.window_mean)

            rmtree(path_tmp)
                                                       
if __name__ == '__main__':
        
#     parser_jaaba_parser = ArgumentParser(parents=[parsers.jaaba_parser])        
    
#     args = parser_jaaba_parser.parse_args()
    
    exit(main())


#!/usr/bin/env python
#
#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
26 Nov 2014

Script to run pergola from the command line using isatab format
"""
from __future__ import print_function
from __future__ import absolute_import

sp = " "

from pergola import parsers
from pergola import isatab_parser
from pergola  import intervals
from pergola  import mapping
# from scripts import pergola_rules
from argparse import ArgumentParser, ArgumentTypeError
from sys      import stderr, exit
from os       import path, makedirs

# from bcbio import isatab
from . import pergola_rules

from urllib2 import urlopen, URLError, HTTPError

home_dir = path.expanduser('~')

path_pergola = path.join(home_dir,".pergola/projects")

if not path.exists(path_pergola):
    makedirs(path_pergola)

def main():

    parser_isatab = ArgumentParser(parents=[parsers.parent_parser])        
    parser_isatab.add_argument('-ft', '--file_tab', required=False, metavar="FILE_TAG", help='Tag for file field in isatab')

    args = parser_isatab.parse_args()

    for input_file in args.input:
        print("@@@Pergola_isatab.py: Input file: %s" % input_file, file=stderr)
        print("@@@Pergola_isatab.py: Configuration file: %s" % args.mapping_file, file=stderr)
        print("@@@Pergola_isatab.py: Selected tracks are: ", args.tracks, file=stderr)
        print("@@@Pergola_isatab.py: Selected tracks are: test", file=stderr)

        # I have to check whether when a isatab folder is given if it is actually a folder or a file
        # difference with -i
        if not path.isdir(input_file):
            raise ValueError ("Argument input must be a folder containing the data in ISAtab format")

        # It might be interesting to check inside the function whether files are url or in path
        dict_files = isatab_parser.parse_isatab_assays (input_file)
        print(dict_files)

        # First try with files in local then with url
        for key in dict_files:
            pointer_file = dict_files[key]

            # Tengo que relacionar de alguna manera cual es el assay de donde tiene que sacar los archivos
            #Probar varios isatab files

            file_path = isatab_parser.check_assay_pointer(pointer_file, download_path=path_pergola)

            pergola_rules.pergola_rules(path=file_path, map_file_path=args.mapping_file,
                               sel_tracks=args.tracks, list=args.list, range=args.range,
                               track_actions=args.track_actions, data_types_list=args.data_types_list,
                               data_types_actions=args.data_types_actions, write_format=args.format,
                               relative_coord=args.relative_coord, intervals_gen=args.intervals_gen,
                               multiply_f=args.multiply_intervals, fields2read=args.fields_read,
                               window_size=args.window_size)

            print("@@@Pergola_isatab.py: : File correctly processed:", file_path, file=stderr)

    print("@@@Pergola_isatab.py: execution finished correctly", file=stderr) 

## It might be interesting to implement an append option

if __name__ == '__main__':
    exit(main())


#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
======================
Module: pergola.tracks
======================

.. module:: tracks

This module  provides the structures to keep the data in genomic format.
It provides a generic class :class:`~pergola.tracks.GenomicContainer` which has 
the general attributes and methods shared by all the other subclasses. 

These subclasses provide special features for each type of data and are:
 
:py:class:`~pergola.tracks.Track` objects are generated by
:func:`pergola.intervals.IntData.read` and hold the parsed and manipulated
data as an iterator and the attributes generated when reading file 

:py:class:`~pergola.tracks.Bed` :class:`~pergola.tracks.GenomicContainer` class 
for Bed files.

:py:class:`~pergola.tracks.BedGraph` :class:`~pergola.tracks.GenomicContainer` 
class for BedGraph files.

"""
from __future__ import print_function

from os         import getcwd
from sys        import stderr, exit
from os.path    import join
from operator   import itemgetter
from itertools  import groupby
from numpy      import arange
import tempfile
from pybedtools import BedTool
from ntpath import split as path_split

## Contains class and file extension
_dict_file = {'bed' : ('Bed', 'track_convert2bed', '.bed'),              
              'bedGraph': ('BedGraph', 'track_convert2bedGraph', '.bedGraph'),
              'gff': ('Gff', 'track_convert2gff', '.gff'),
              'txt': ('Track', '', '.txt')}

# From light to dark
# n_interval = 9

# _black_gradient = ["226,226,226", "198,198,198", "170,170,170", "141,141,141", "113,113,113", "85,85,85", "56,56,56", "28,28,28", "0,0,0"]
# _blue_gradient = ["224,255,255", "173,216,230", "135,206,250", "135,206,235", "65,105,225", "0,0,255", "0,0,205", "0,0,139", "0,0,128"] 
# _red_gradient = ["254,172,182", "254,153,162", "254,134,142", "254,115,121", "254,96,101", "254,77,81", "254,57,61", "254,38,40", "254,19,20"]
# _green_gradient = ["203,254,203", "178,254,178", "152,254,152", "127,254,127", "102,254,102", "76,254,76", "51,254,51", "0,254,0", "25,115,25"]
# _orange_gradient = ["255,223,0", "255,240,0", "255,220,0", "255,200,0", "255,180,0", "255,160,0", "255,140,0", "250,115,0", "255,100,0"]

n_interval = 4

## short gradients
_black_gradient = ["170,170,170", "113,113,113", "85,85,85", "56,56,56", "28,28,28", "0,0,0"]
_blue_gradient = ["135,206,250", "65,105,225", "0,0,255", "0,0,205", "0,0,139", "0,0,128"] 
_red_gradient = ["254,134,142", "254,96,101", "254,77,81", "254,57,61", "254,38,40", "254,19,20"]
_green_gradient = ["152,254,152", "102,254,102", "76,254,76", "51,254,51", "0,254,0", "25,115,25"]
_orange_gradient = ["255,220,0", "255,180,0", "255,160,0", "255,140,0", "250,115,0", "255,100,0"]
_yellow_gradient = ["255,255,0", "255,255,31", "255,255,51", "255,255,102", "255,255,153", "255,255,204"]
_pink_gradient = ["229,0,174", "234,38,190", "239,77,206", "244,116,222", "246,135,230", "254,194,254"]
_cyan_gradient = ["0,186,229", "38,199,234", "58,205,236", "116,225,244", "135,231,246", "174,244,251"]

_dict_colors = {
                'black' : _black_gradient,
                'blue' : _blue_gradient,
                'red' : _red_gradient,
                'green' : _green_gradient,
                'orange' : _orange_gradient,
                'yellow' : _yellow_gradient,
                'pink' : _pink_gradient,
                'cyan': _cyan_gradient
                }

# _intervals = [0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 1, 1000] #del

_max_file_name_len = 100


class GenomicContainer(object):
    """
    This class provides the general attributes and methods shared by all the other
    subclasses that contain the data after being read :func:`pergola.intervals.IntData.read`
    or converted by :py:func:`pergola.tracks.Track.convert`

    .. attribute:: data
    
       Iterator yielding record of the genomic data
    
    .. attribute:: fields
    
       List of name of each of the fields contained in records of data
    
    .. attribute:: format
       
       Format to write file (extension) 
    
    .. attribute:: track
       
       String with the name of the track
    
    .. attribute:: range_values
       
       Range of values inside data_value field
       
    ..
       Indicates the presence of a header.
       * `False` if there is no header. Fields should the be provided using fields param
       * `True` if the file have a header line with names. This names should match names in ontology_dict (default).
       
    :returns: GenomicContainer object
        
    """

    def __init__(self, data, fields=None, data_types=None, **kwargs):
        if isinstance(data,(tuple)):            
            data = iter(data)
        
        if not fields:
            raise ValueError("Must specify a 'fields' attribute for %s." % self.__str__())
        
        self.data = data
        self.fields = fields
        self.data_types = data_types     
        self.format = kwargs.get("format",'txt')
        self.track = kwargs.get('track', "1")
        self.range_values = kwargs.get('range_values', None)
        
    def __iter__(self):
        return self.data

    def next(self):
        """        
        :returns: next item in iterator
         
        """

        return next(self.data)
    
    def save_track(self, mode="w", path=None, name_file=None, track_line=True, bed_label=False, gff_label=False):
        """
        Save the data in a file of format set by *self.format* 
        
        :param "w" mode: :py:func:`str` Only implemented write 'w' (default). 
        
        :param None path: Path to create file, py:func:`str`. If None (default) the 
            file is dumped in the current working directory and prints a warning.  
        
        :param None track: Path to create file, py:func:`str`. If None (default) the 
            file is dumped in the current working directory and prints a warning.
            
        :param True track_line: If it is set to True includes the track_line 
        
        :param None name_file: :py:func: `str` to set name of output file
        
        :param False bed_label: Whether to include or not the labels of each interval, 
            default False in bed files
        
        :returns: Void
        
        """
        
        if not path: 
            pwd = getcwd()
            print("No path selected, files dump into path: ", pwd, file=stderr) 
        else:
            pwd = path
            print("Files dump into path: ", pwd, file=stderr)
         
                             
        if not(isinstance(self, GenomicContainer)):
            raise Exception("Not writable object, type not supported '%s'."%(type(self)))    
        
        try:
            file_ext = _dict_file.get(self.format)[2]      
        except KeyError:
            raise ValueError("File types not supported \'%s\'"%(self.format))
        
        if name_file is None:
            conc_data_types = self.data_types
            if isinstance(conc_data_types, set):
                conc_data_types="_".join(self.data_types)        
            
            if len ("tr_" + self.track + "_dt_" + conc_data_types + file_ext) < _max_file_name_len: 
                name_file = "tr_" + self.track + "_dt_" + conc_data_types + file_ext
            else:
                name_file = "tr_" + self.track + "all_data_types" +  file_ext 
                
        else:
            if not name_file.endswith('.tmp'):
                name_file = name_file + file_ext
                
        print("File %s generated" % name_file, file=stderr)       

        track_file = open(join(pwd, name_file), mode)
                
        ## Annotation track to set the genome browser interface
        annotation_track = ''
        data_out = []
        
        if self.format == 'bed' and track_line:
            annotation_track = 'track ' + 'name=\"' +  self.track + "_" + self.data_types + '\"' + " " + 'description=\"' + self.track + " " + self.data_types + '\"' + " " + "visibility=2 itemRgb=\"On\" priority=20"
            track_file.write (annotation_track + "\n")
            
        elif self.format == 'bedGraph' and track_line:
            annotation_track = 'track ' + 'name=\"' + self.track + "_" + self.data_types + '\"' + " " + 'description=\"' + self.track + "_" + self.data_types + '\"' + " " + 'visibility=full color=' + self.color_gradient[n_interval-1] + ' altColor=' + self.color_gradient[n_interval] + ' priority=20'        #                         
            track_file.write (annotation_track + "\n")        
        
        if self.format == 'gff':
            file_format_line = '##gff-version 3'
            track_file.write (file_format_line + "\n")
            track_file.write ('##sequence-region 1' + "\t" + "1"  "\t" + "1" + "\t" + "50" +  "\n")

        data_out = sorted(self.data, key=itemgetter(self.fields.index('start')))
                
        # for row in self.data:
        for row in data_out:  
            
            if self.format == 'bed' and not bed_label:
                index_name = self.fields.index ('name')                 
                empty_label = "."
                row = [empty_label if (i == index_name) else (v) for (i, v) in enumerate(row)]
            
            elif self.format == 'gff' and not bed_label:
                index_name = self.fields.index ('feature')                 
                empty_label = "."
                row = [empty_label if (i == index_name) else (v) for (i, v) in enumerate(row)]
                    
            track_file.write('\t'.join(str(v) for v in row))           
            track_file.write("\n")
                  
        track_file.close()
    
    
#     def _tmp_bed(self):
#         tmp_bed = tempfile.NamedTemporaryFile(prefix='pergola.',
#                                             suffix='.tmp',
#                                             delete=False)
#         tmp_bed = tmp_bed.name
#         return tmp_bed
    
    # Quiza mejor hacerlo fuera porque asi puedo hacer excepciones en plan no se ha instaldo pybedtools
    # sino tengo dependencia con pybedtools 
#     def create_pybedtools(self): #del
#         f_bed = self._tmp_bed()
#         path_bed, name_bed = path_split(f_bed)
#         self.save_track(path = path_bed, name_file=name_bed, bed_label=True)
#         return BedTool(f_bed)
    
    
class Track(GenomicContainer):
    """
    This class contains all the methods to operate and convert the data list into the 
    different possible classes
    
    .. attribute:: data

       Iterator yielding record of the genomic data

    .. attribute:: fields

       List of name of each of the fields contained in records of data

    .. attribute:: data_types

       List of data types contained in the data 

    .. attribute:: list_tracks

       List of tracks contained in the data 

    .. attribute:: min

       Minimum value contained in the data
    
    .. attribute:: max

       Maximum value contained in the data
    
    :returns: Dictionary of Bed or BedGraph objects
    
    """

    def __init__(self, data, fields=None, data_types=None, list_tracks=None, min=0, max=0, **kwargs):
        self.list_tracks = list_tracks
        self.list_tracks_filt = []
        self.list_data_types = data_types
        self.min = min
        self.max = max

        GenomicContainer.__init__(self, data, fields, data_types, **kwargs)
        
    def convert(self, mode='bed', range_color=None, **kwargs):
        """
        Calls function to convert data (as a list of tuples) into a dictionary of 
        the one or several object of class set by mode
        
        :param bed mode: class of the output objects returned set to `bed` by
            default
            
        :param range_color: :py:func:`list` with range of values to set colors 
            of bed file. If set modifies self.range_values
            
        :param tracks2remove: :py:func:`list` of tracks to remove from the dict_t
        
        :returns: dictionary containing object/s of the class set by mode 
        
        """

        kwargs['relative_coord'] = kwargs.get("relative_coord", False)
        
        if mode not in _dict_file: 
            raise ValueError("Mode \'%s\' not available. Possible convert() modes are %s"%(mode,', '.join(['{}'.format(m) for m in _dict_file.keys()])))
        
        # User set values for bed file colors
        if range_color:
            if (len(range_color) != 2 or not(all(isinstance(n, (int, float)) for n in range_color))):
                raise ValueError ("Range color must be a list with two numeric values" \
                                % (range_color))
        
            self.range_values = range_color

        dict_tracks = (self._convert2single_track(self.data, mode, **kwargs)) 
        
        return (dict_tracks)
        
    def _convert2single_track(self, data_tuples,  mode=None, **kwargs):
        """
        Split data (as a list of tuples) into one or several objects depending on options 
        selected. Each object will correspond to a track in the genome browser
        
        :param data_tuples: list of tuples containing data
        :param None mode: class of each single track that will be hold in dictionary
            
        :returns: dictionary of tracks
        
        """
           
        dict_split = {}
        
        ### Data is separated by track and data_types
        idx_fields2split = [self.fields.index("track"), self.fields.index("data_types")]
        data_tuples = sorted(data_tuples, key=itemgetter(*idx_fields2split))
        
        for key,group in groupby(data_tuples, itemgetter(*idx_fields2split)):
            if key[0] not in dict_split:
                dict_split [key[0]] = {}
            dict_split [key[0]][key[1]] = tuple(group)
        
        #Generates dictionary of original fields and color gradients
        color_restrictions = kwargs.get('color_restrictions', None)
        _dict_col_grad = dict()
        
        self.list_tracks_filt = self.list_tracks
        
        ### Tracks not set in tracks option are filtered out
        sel_tracks = []
        if not kwargs.get('tracks'):
            pass
        else:
            sel_tracks = map(str, kwargs.get("tracks",[]))
                   
        ### When any tracks are selected we consider that no track should be removed
        if sel_tracks != []:            
            ori_tracks = set(self.list_tracks)
            tracks2rm = self.list_tracks.difference(sel_tracks)
#             tracks2rm = sel_tracks                           
            dict_split = self.remove (dict_split, tracks2rm)
            print("Removed tracks are:", ' '.join(sorted(tracks2rm, key=int)), file=stderr)
            
            # I have to keep the original list otherwise the original object is changed
            self.list_tracks = ori_tracks
            
        # Eventually I can eliminate tracks of not selected tracks
#         self.data = [y for y in data_tuples if y[0] in self.list_tracks]
        
        sel_data_types = []
        if not kwargs.get('data_types'):
            pass
        else:            
            sel_data_types = map(str, kwargs.get("data_types",[]))
        
        new_dict_split = {} 
           
        ### When any data_types are selected we consider that no data_types should be removed
        if sel_data_types  != []:
            data_types2rm = self.list_data_types.difference(sel_data_types)
            for track, track_dict in dict_split.items():    
                dict_data_type = self.remove (track_dict, data_types2rm)        
                new_dict_split [track] = dict_data_type
            print("Removed data types are:", ' '.join(data_types2rm), file=stderr)        

        if new_dict_split: 
            dict_split = new_dict_split
                
        d_track_merge = {} 
        
        ### If tracks_merge is set we combine tracks selected                 
        if not kwargs.get('tracks_merge'):
            d_track_merge = dict_split
        else:
#             tracks_merge = kwargs.get('tracks_merge',self.list_tracks)
            tracks_merge = kwargs.get('tracks_merge',self.list_tracks_filt)
#             if not all(tracks in self.list_tracks for tracks in tracks_merge):
            if not all(tracks in self.list_tracks_filt for tracks in tracks_merge):
                raise ValueError ("Tracks to merge: %s, are not in the track list: " % 
                                  ",".join("'{0}'".format(n) for n in tracks_merge), 
                                  ",".join("'{0}'".format(n) for n in self.list_tracks_filt))
            print(("Tracks that will be merged are: %s" %  " ".join(tracks_merge)), file=stderr)
            
            d_track_merge = self.join_by_track(dict_split, tracks_merge)       
        
        d_data_types_merge = {}
        
        ### If set we join the data_types or natures
        if not kwargs.get('data_types_actions') or kwargs.get('data_types_actions') == 'one_per_channel':
            d_data_types_merge = d_track_merge
        elif kwargs.get('data_types_actions') == 'all':
            d_data_types_merge = self.join_by_dataType(d_track_merge, mode)
        
        if mode == 'bedGraph':     
            _dict_col_grad = assign_color (self.data_types, color_restrictions)
         
        track_dict = {}                        
   
        ### Generating track dict (output)                         
        window = kwargs.get("window", 300)
        mean_win = kwargs.get("mean_win", False)
        mean_value = kwargs.get("mean_value", False)

        ### Assigning data to output dictionary
        for k, d in d_data_types_merge.items():
            if not isinstance(d,dict):
                raise ValueError ("The structure that holds the tracks should be a dictionary of dictionaries")
            
            for k_2, d_2 in d.items():
                if not k_2 in _dict_col_grad and mode == "bed" or mode == "gff":
                    _dict_col_grad[k_2] = ""
                
                range_val = self._get_range(d_2)

                i_chr_start = self.fields.index("start")
                i_chr_end = self.fields.index("end")
                min_time = kwargs.get('min_time', self.min)
                max_time = kwargs.get('max_time', self.max)

                d_2 = [r for r in d_2 if r[i_chr_start] >= min_time and r[i_chr_end] <= max_time]

                if d_2:
                    track_dict[k,k_2] = globals()[_dict_file[mode][0]](getattr(self,_dict_file[mode][1])(d_2,
                                                                                                         True,
                                                                                                         window=window,
                                                                                                         mean_win=mean_win,
                                                                                                         mean_value=mean_value,
                                                                                                         color_restrictions=color_restrictions,
                                                                                                         min_t = self.min,
                                                                                                         max_t = self.max,
                                                                                                         min_time=kwargs.get('min_time', self.min),
                                                                                                         max_time=kwargs.get('max_time', self.max)),
                                                                                                         track=k, data_types=k_2,
                                                                                                         range_values=range_val,
                                                                                                         color=_dict_col_grad[k_2])
                else:
                    print(("WARNING: Processed track %s, %s will be deleted because is empty with current " \
                                      "time boundaries." % (k, k_2)), file=stderr)

        return track_dict
    
    def _get_range(self, data_tr):
        """
        Calculates the range of values in data_value field 
        
        :param data_tr: :py:func:`tuple` of tuples containing data of a single track    
                
        :returns: range_list list with minimum and maximum value in data

        """

        try:
            i_data_value = self.fields.index("data_value")
        except KeyError:
            raise ValueError("Data value index is not set")
        
        min = -10000000
        max = -10000000
        
        for r in data_tr:
            v = r[i_data_value]
            
            if min == -10000000: min = v
            if min > v: min = v
            
            if max == -10000000: max = v
            if max < v: max = v
            
        range_list = [min, max]
           
        return range_list
    
    def remove(self, dict_t, tracks2remove):
        """
        Removes selected tracks from a dictionary of tracks that is the input of the function those that are 
        set by tracks2remove
        
        :param dict_t: py:func:`dict` dictionary containing one or more tracks, keys represent each 
            of these tracks
        :param tracks2remove: :py:func:`list` of tracks to remove from the dict_t
             
        :returns: dict_t dictionary, that contains the remaining tracks
        
        #TODO I can make this function more general as remove from dictionary it can be use outside
        # in fact I am using now it to remove data_types and not only tracks
        
        """

        for key in tracks2remove:
            key = str(key)
    
            dict_t.pop(key, None)
            
#             if key in self.list_tracks:        #OJO
#                 self.list_tracks.remove(key)
            
            if key in self.list_tracks_filt:        
                self.list_tracks_filt.remove(key)
            
#             if key in self.list_data_types:
#                 self.list_data_types.remove(key)
                    
        return (dict_t)
    
    def join_by_track(self, dict_t, tracks2join):  
        """
        Join tracks by track name or id 
        
        :param dict_t: py:func:`dict` containing one or more tracks, keys 
            represent each of these tracks
        :param tracks2join: :py:func:`list` of tracks to join in a single track
             
        :returns: d_track_merge dictionary that contains the joined tracks
        
        TODO What if I give to the function only some tracks to join, what happen
            to the remaining tracks
         
        """
        
        d_track_merge = {} 
        new_tracks = set()
        
        for key, nest_dict in dict_t.items():
            
            if key not in tracks2join: 
                print("Track not use because was not set when join_by_track is called: %s" % key)
                continue
            
            if '_'.join(tracks2join) not in d_track_merge:
                d_track_merge['_'.join(tracks2join)] = {}
                new_tracks.add('_'.join(tracks2join))
            
            for key_2, data in nest_dict.items():                            
                if key_2 not in d_track_merge['_'.join(tracks2join)]:
                    d_track_merge['_'.join(tracks2join)] [key_2]= data
                else:  
                    d_track_merge['_'.join(tracks2join)] [key_2] = d_track_merge['_'.join(tracks2join)] [key_2] + data

#         self.list_tracks = new_tracks
                   
        return (d_track_merge)
    
    
    def join_by_dataType (self, dict_d, mode):
        """
        Join tracks by dataType
        
        :param dict_d: py:func:`dict` containing one or more tracks, primary key 
            are tracks id and secondary tracks are data_types
        :param mode: :py:func:`str` class of the object that is going to be 
            generated
             
        :returns: d_data_types_merge dictionary that contains the joined tracks and
                  the new dictionary with colors assign by data type
         
        """
        d_data_types_merge = {}
        new_data_types = set()
        
        for key, nest_dict in dict_d.items():
            
            d_data_types_merge[key] = {}                        
            
            for key_2, data in nest_dict.items(): 
                
                if '_'.join(nest_dict.keys()) not in d_data_types_merge[key]:
                    d_data_types_merge[key]['_'.join(nest_dict.keys())] = data                    
                    new_data_types.add('_'.join(nest_dict.keys())) 
                else:                    
                    d_data_types_merge[key]['_'.join(nest_dict.keys())] = d_data_types_merge[key]['_'.join(nest_dict.keys())] + data
                    new_data_types.add('_'.join(nest_dict.keys()))          
        
        # New data_types only set if objects is bedGraph. Bed objects needs to
        # know all original data_types to display them with different colors
        if mode == 'bedGraph':
            self.data_types = new_data_types
            
        return (d_data_types_merge)
    
    def track_convert2bed(self, track, in_call=False, **kwargs):
        """
        Converts data belonging to a single track (in the form of a list of tuples) in
        an object of class Bed
        
        :param track: :py:func:`list` of tuples containing data of a single track
        :param False in_call: If False the call to the function is from the user otherwise
            is from inside :py:func: `convert2single_track()`
        :param None color_restrictions: Set colors not to be used #TODO this is not clear example??        
                
        :returns: Bed object
        
        """        

        # This fields are mandatory in objects of class Bed
        _bed_fields = ["track","start","end","data_types", "data_value"]        
        
        # Check whether these fields are in the original otherwise raise exception
        try:
            [self.fields.index(f) for f in _bed_fields]
        except ValueError:
            raise ValueError("Mandatory field for Bed object creation '%s' is missing." % (f))

#         if (not in_call and len(self.list_tracks) != 1):
        if (not in_call and len(self.list_tracks_filt) != 1):
            raise ValueError("Your file '%s' has more than one track, only single tracks can be converted to bed" % (self.path))
        
        i_track = self.fields.index("track")
        i_chr_start = self.fields.index("start")
        i_chr_end = self.fields.index("end")
        i_data_value = self.fields.index("data_value")
        i_data_types = self.fields.index("data_types")
        
        #Generate dictionary of field and color gradients
        color_restrictions = kwargs.get('color_restrictions', None)
        _dict_col_grad = assign_color (self.data_types, color_restrictions)
        
        step = (float(self.range_values[1]) - float(self.range_values[0])) / n_interval

        if step == 0: 
            _intervals = [0, self.range_values[1]]
        else: 
            _intervals = list(arange(float(self.range_values[0]), float(self.range_values[1]), step))

        for row in track:
            temp_list = []
            temp_list.append("chr1")
            temp_list.append(row[i_chr_start])
            temp_list.append(row[i_chr_end])
            temp_list.append(row[i_data_types])  
            temp_list.append(row[i_data_value])   
            temp_list.append("+")
            temp_list.append(row[i_chr_start])
            temp_list.append(row[i_chr_end])
            
            if step != 0:
                for i,v in enumerate(_intervals):    
                    d_type = row [self.fields.index("data_types")]
                    global color
                    color = _dict_col_grad[d_type][len(_intervals)-1]

                    if float(row[i_data_value]) <= v:                    
                        color = _dict_col_grad[d_type][i]                               
                        break
            else:
                 d_type = row [self.fields.index("data_types")]
                 # global color
                 color = _dict_col_grad[d_type][-1]
                 
            temp_list.append(color)          
            
            yield(tuple(temp_list))
    
    def track_convert2gff(self, track, in_call=False, **kwargs):
        """
        Converts data belonging to a single track (in the form of a list of tuples) in
        an object of class Gff
        
        :param track: :py:func:`list` of tuples containing data of a single track
        :param False in_call: If False the call to the function is from the user otherwise
            is from inside :py:func: `convert2single_track()`
        :param None color_restrictions: Set colors not to be used #TODO this is not clear example??        
                
        :returns: Gff object
        
        """        

        #This fields are mandatory in objects of class Bed
#         _bed_fields = ["track","chrom_start","chrom_end","data_types", "data_value"]        

#         _gff_fields = ['seqname','source','feature','start','end','score', 'strand','frame','attribute']
        
        ## Following specifications of:
        ## http://gmod.org/wiki/GFF3
#         _bed_fields = ["track","chrom_start","chrom_end","data_types", "data_value"]        
        
#         _gff_fields_mand = ['seqname','source','type','start','end','score', 'strand','frame','attribute']
        _gff_fields_mand = ["track", "data_types", "start","end", "data_value"] 
#         _gff_fields_mand = ["seqname", "feature", "chrom_start", "chrom_end", "score"]
            
        '''
        'seqname' track
        'source'  NA NOT MANDATORY "." (a period) in this field.
        'type' "data_types"
        'start'   "start" REQUIRED
        'end'     "end"   REQUIRED
        'score'   "data_value" Not required
        'strand'  NA AND NOT MANDATORY "." (a period) in this field.
        'frame'   NA AND NOT MANDATORY "." (a period) in this field.
        'attribute'  Not required 
        http://gmod.org/wiki/GFF3
        ''' 
        #Check whether these fields are in the original otherwise raise exception
        try:
            [self.fields.index(f) for f in _gff_fields_mand]
        except ValueError:
            raise ValueError("Mandatory field for Gff object creation '%s' is missing." % (f))

#         if (not in_call and len(self.list_tracks) != 1):
        if (not in_call and len(self.list_tracks_filt) != 1):
            raise ValueError("Your file '%s' has more than one track, only single tracks can be converted to bed" % (self.path))
        """
        i_track = self.fields.index("track")
        i_chr_start = self.fields.index("start")
        i_chr_end = self.fields.index("end")
        i_data_value = self.fields.index("data_value")
        i_data_types = self.fields.index("data_types")
        """

#         i_seqname = self.fields.index("seqname")
#         i_types = self.fields.index("feature")        
#         i_start = self.fields.index("chrom_start")
#         i_end = self.fields.index("chrom_end")        
#         i_score = self.fields.index("score")        
        i_seqname = self.fields.index("track")
        i_types = self.fields.index("data_types")        
        i_start = self.fields.index("start")
        i_end = self.fields.index("end")        
        i_score = self.fields.index("data_value") 
        
        #Generate dictionary of field and color gradients
        color_restrictions = kwargs.get('color_restrictions', None)
        _dict_col_grad = assign_color (self.data_types, color_restrictions)
        
        # Step for the color gradient
        step = (float(self.range_values[1]) - float(self.range_values[0])) / n_interval

        if step == 0: 
            _intervals = [0, self.range_values[1]]
        else: 
            _intervals = list(arange(float(self.range_values[0]), float(self.range_values[1]), step))
        
        for row in track:
            temp_list = []
#             temp_list.append(row[i_seqname]) # "seqid"
            temp_list.append(1)  # "seqid"
            temp_list.append(".")  # "source"
#             temp_list.append(row[i_types]) #"type"
            temp_list.append("exon")  # "type"
            temp_list.append(row[i_start] + 1)  # start
            temp_list.append(row[i_end] + 1)  # end
            temp_list.append(row[i_score])  # "score"
            temp_list.append(".")  # "strand"
            temp_list.append(".")  # phase
            
            if step != 0:
                for i,v in enumerate(_intervals):    
                    d_type = row [self.fields.index("data_types")]
                    global color
                    color = _dict_col_grad[d_type][len(_intervals)-1]

                    if float(row[i_score]) <= v:                    
                        color = _dict_col_grad[d_type][i]                               
                        break
            else:
                 d_type = row [self.fields.index("data_types")]
                 # global color
                 color = _dict_col_grad[d_type][-1]
                 
            temp_list.append("color=" + color)          
            
            yield(tuple(temp_list))
            
    def track_convert2bedGraph(self, track, in_call=False, window=300, mean_win=False, mean_value=False,  **kwargs):
        """
        Converts a single data belonging to a single track in a list of tuples in
        an object of class BedGraph. The data is grouped in time windows.
            
        :param track: :py:func:`list` of tuples containing data of a single track
        :param False in_call: If False the call to the function is from the user otherwise
            is from inside :py:func: `convert2single_track()`
        :param window: :py:func:`int` length of windows inside bedGraph file in seconds (default 300)
        :param False mean_win: Calculates average value over the time interval set by window (sum values/length window)
        :param False mean_value: Calculates average value (sum values/count)
        
        :returns: BedGraph object
        """

        #This fields are mandatory in objects of class BedGraph
        _bed_fields = ["track","start","end","data_value"] 
        
        #Check whether these fields are in the original otherwise raise exception
        try:
            idx_f = [self.fields.index(f) for f in _bed_fields]                          
        except ValueError:
              raise ValueError("Mandatory field for BedGraph object creation '%s' is missing." % (f))
        
#         if (not in_call and len(self.list_tracks)  != 1):            
        if (not in_call and len(self.list_tracks_filt)  != 1):
#             raise ValueError("Your file '%s' has more than one track, only single tracks can be converted to bedGraph" % (self.path))
            raise ValueError("Your data has more than one track, only single tracks can be converted to bedGraph.")
        
        i_track = self.fields.index("track")
        i_chr_start = self.fields.index("start")
        i_chr_end = self.fields.index("end")
        i_data_value = self.fields.index("data_value")
        
        ## When the tracks have been join it is necessary to order by chr_start
        track = sorted(track, key=itemgetter(*[i_chr_start]))
        # min_time = kwargs.get('min_time', self.min)
        # max_time = kwargs.get('max_time', self.max)

        ## Filtering by min and max time points
        # track = [row for row in track if row[i_chr_start] >= min_time and row[i_chr_end] <= max_time]

        # Dumping raw data as a bedGraph file, no binning
        if not window or window == 0:  # or false

            for row in track:
                temp_list = []
                temp_list.append("chr1")
                temp_list.append(row[i_chr_start])
                temp_list.append(row[i_chr_end])
    #             temp_list.append(row[i_data_types])
                temp_list.append(row[i_data_value])
    #             temp_list.append("+")
    #             temp_list.append(row[i_chr_start])
    #             temp_list.append(row[i_chr_end])

    #             temp_list.append(color)

                yield(tuple(temp_list))
        
        # binning in windows of time     
        else:
            try:                
                float(window)
            except:               
                raise ValueError("Window option only accepts integers or False. Current value: %s" 
                                % (type(window)))
            
            if float(window) != int(window):
                raise ValueError("Window option only accepts integers or False. Current value: %s" 
                                % (type(window)))
                
            # Ini_window has to be set to the initial value otherwise files with            
            # no relative coordinates contained tones of empty data
    #         ini_window = 0 #ojo
    #         ini_window = 1       
            # I have to find the closest number multiple of the window size so that all 
            # bedGraph have the same intervals
                        
            delta_window = window

            # To read files for instance in GRanges objects all tracks same number of intervals              
            # ini_window_def = divmod(track[0][i_chr_start]/delta_window, 1)[0] * delta_window
            min_t = kwargs.get('min_t', self.min)
            max_t = kwargs.get('max_t', self.max)

            min_time = kwargs.get('min_time', min_t)
            max_time = kwargs.get('max_time', max_t)

            if min_time is not None:
                min_t = min_time
                if min_time < min_t:
                    print(("WARNING: min_time \'%d\' is smaller than minimun time point \'%d\' inside the input file" %(min_time, min_t)), file=stderr)

            if max_time is not None:
                max_t = max_time
                if max_time > max_t:
                    print(("WARNING: max_time \'%d\' is bigger than minimun time point \'%d\' inside the input file" %(max_time, max_t)), file=stderr)

            ini_window = divmod(min_t/delta_window, 1)[0] * delta_window
            end_window = ini_window + delta_window
                        
            partial_value = 0
            counts = 0
            mean_value_l = []
            cross_interv_dict = {}
            
#             last_point =  track[-1][i_chr_end]
            last_point = max_t
            r = last_point % delta_window
            fake_end = last_point # + delta_window - r

            if track:

                last_fake_line = list(track[-1])

                if last_fake_line[i_chr_end] > max_t + 1:
                    exit("FATAL ERROR: Something went wrong during bedGraph window conversion")

                last_fake_line[i_chr_start] = last_fake_line[i_chr_end] + 1
                last_fake_line[i_chr_end] = fake_end
                last_fake_line[i_data_value] = 0
                track.append(tuple(last_fake_line))

                if not r == 1 or r == 0:
                    fake_end = last_point + delta_window - r
                    sec_fake_line = last_fake_line
                    sec_fake_line[i_chr_start] = fake_end + delta_window + 1
                    sec_fake_line[i_chr_end] = fake_end + 2 * delta_window
                    sec_fake_line[i_data_value] = 0
                    track.append(tuple(sec_fake_line))

                for row in track:
                    temp_list = []
                    chr_start = row[i_chr_start]
                    chr_end = row[i_chr_end]
                    data_value = float(row[i_data_value])
                    f = "data_value"
                    self.fields.index(f)

                    # Intervals happening after the current window
                    # if there is a value accumulated it has to be dumped otherwise 0
                    if chr_start > end_window:
                        while end_window < chr_start:
                            partial_value = partial_value + cross_interv_dict.get(ini_window, 0)

                            if mean_value and cross_interv_dict.get(ini_window):
                                mean_value_l.append(cross_interv_dict.get(ini_window, 0))

                            temp_list.append("chr1")
                            temp_list.append(ini_window)
                            temp_list.append(end_window)

                            if mean_win:
                                temp_list.append(partial_value/window)
                            elif mean_value and counts:
                                if float(sum(mean_value_l)) == 0:
                                    temp_list.append(0)
                                else:
                                    temp_list.append(float(sum(mean_value_l)) / float(len(mean_value_l)))
                                # else:
                                #     temp_list.append(partial_value)
                            else:
                                temp_list.append(partial_value)

                            partial_value = 0
                            counts = 0
                            mean_value_l = []
        #                     ini_window += delta_window + 1 #ojo
                            ini_window += delta_window
        #                     end_window += delta_window + 1 #ojo
                            end_window += delta_window

                            if end_window > max_t:
                                break

                            yield(tuple(temp_list))
                            temp_list = []

                        ## Value must to be weighted between intervals
                        if chr_end > end_window:
                            value2weight = data_value
                            end_w = end_window
                            start_new = chr_start
                            end_new = chr_end

                            for start_w in range (int(ini_window), int(chr_end), delta_window):
                                weighted_value = 0.0
                                counts += 1

                                if end_w == start_w:
                                    weighted_value = float(end_w - start_new + 1) / float(end_new - start_new)
                                elif end_new == start_new:
                                    break
                                else:
                                    weighted_value = float(end_w - start_new) / float(end_new - start_new)
                                    # weighted_value= 9/2

                                weighted_value *= value2weight
                                cross_interv_dict[start_w] = float(cross_interv_dict.get(start_w, 0)) + float(weighted_value)
                                start_new = end_w
                                value2weight = value2weight - weighted_value

                                if (end_w + delta_window) >= chr_end:
                                    new_start_w = start_w + delta_window
                                    cross_interv_dict[new_start_w] = cross_interv_dict.get(new_start_w, 0) + value2weight
                                    break

                                end_w = end_w + delta_window
                        else:
                            partial_value = partial_value + data_value

                            if mean_value:
                                counts += 1
                                mean_value_l.append(data_value)

                    # elif chr_start <= end_window and chr_start >= ini_window:
                    elif end_window > chr_start >= ini_window:
                        if chr_end <= end_window:
                            partial_value = partial_value + data_value

                            if mean_value and data_value:
                                mean_value_l.append(data_value)
                                counts += 1

                        else:
                            value2weight = data_value
                            end_w = end_window
                            start_new = chr_start
                            end_new = chr_end

                            for start_w in range (int(ini_window), int(chr_end), delta_window):
                                weighted_value = 0
                                counts += 1

                                if end_w == start_w:
                                    weighted_value = float(end_w - start_new + 1) / float(end_new - start_new)
                                else:
                                    weighted_value = float(end_w - start_new) / float(end_new - start_new)

                                weighted_value *= value2weight
                                cross_interv_dict[start_w] = float(cross_interv_dict.get(start_w,0)) + float(weighted_value)
                                start_new = end_w
                                value2weight = value2weight - weighted_value

                                if end_w + delta_window >= chr_end:
                                    new_start_w = start_w + delta_window
                                    cross_interv_dict[new_start_w] = cross_interv_dict.get(new_start_w,0) + value2weight
                                    break

                                end_w = end_w + delta_window

                    elif chr_start < ini_window:
                        print(("WARNING: Value %d deleted because you set first time point " \
                                          "to a higher value %d") % (chr_start, end_window), file=stderr)
                # else:
                    # print >> stderr, ("FATAL ERROR: Something went wrong during bedGraph window " \
                    #                   "conversion.")

        # Last value just printed out
#         temp_list.append("chr1")        
#         temp_list.append(ini_window)
#         temp_list.append(end_window)
#         temp_list.append(data_value)
#         yield(tuple(temp_list))

class BedToolConvertible(GenomicContainer):
    def __init__(self, data, **kwargs):
        # GenomicContainer.__init__(self,data,**kwargs)
        # GenomicContainer.__init__(self, data, fields, data_types, **kwargs)
        GenomicContainer.__init__(self, data, **kwargs)
    
    def create_pybedtools(self):
        """
        Converts a single data belonging to a single track in a list of tuples in
        an object of class BedGraph. The data is grouped in time windows.
            
        :param track: :py:func:`list` of tuples containing data of a single track
        :param False in_call: If False the call to the function is from the user otherwise
            is from inside :py:func: `convert2single_track()`
        :param window: :py:func:`int` length of windows inside bedGraph file in seconds (default 300)
                 
        :returns: BedGraph objectx
        """
        track_tmp = self._tmp_track()
        
        if not(isinstance(self, GenomicContainer)):
            raise Exception("Object \'%s\' can not be converted to BedTool" % (type(self)))    
        
        try:
            file_ext = _dict_file.get(self.format)[2]      
        except KeyError:
            raise ValueError("File types \'%s\' not convertible to BedTool" % (self.format))
        
        data_tmp = []
        data_tmp = sorted(self.data, key=itemgetter(self.fields.index('start')))
        
        with open(track_tmp, "w") as bedtool_out:         
            for r in data_tmp:  
                bedtool_out.write('\t'.join(str(f) for f in r))           
                bedtool_out.write("\n")
        
        bedtool_obj = BedTool(track_tmp)            
        
        return(bedtool_obj)
        
    def _tmp_track(self):
        """
        TODO: comment
        """
        tmp_track = tempfile.NamedTemporaryFile(prefix='pergola.', suffix='.tmp', delete=False)
        tmp_track = tmp_track.name
        
        return tmp_track    
            
        
                              
# class Bed(GenomicContainer):
class Bed(BedToolConvertible):
    """
    A :class:`~pergola.tracks.GenomicContainer` object designed to include specific 
    fields and features of **bed files**
    
    Default fields are
        ::
        
         ['chr','start','end','name','score','strand',
          'thick_start','thick_end','item_rgb']
    
    :returns: Bed object    
        
    """
    def __init__(self, data, **kwargs):
        kwargs['format'] = 'bed'
        
        # 
        kwargs['fields'] = ['chr','start','end','name','score','strand',
                            'thick_start','thick_end','item_rgb']
        
#         GenomicContainer.__init__(self,data,**kwargs)
        BedToolConvertible.__init__(self,data,**kwargs)
        
        
#     def _tmp_bed(self):
#         tmp_bed = tempfile.NamedTemporaryFile(prefix='pergola.',
#                                             suffix='.tmp',
#                                             delete=False)
#         tmp_bed = tmp_bed.name
#         return tmp_bed
#     
#     # Quiza mejor hacerlo fuera porque asi puedo hacer excepciones en plan no se ha instaldo pybedtools
#     # sino tengo dependencia con pybedtools 
#     def create_pybedtools(self):
#         f_bed = self._tmp_bed()
#         path_bed, name_bed = path_split(f_bed)
#         self.save_track(path = path_bed, name_file=name_bed, bed_label=True)
#         return BedTool(f_bed)

class BedGraph(BedToolConvertible):
# class BedGraph(GenomicContainer):
    """
    A :class:`~pergola.tracks.GenomicContainer` object designed to include specific 
    fields and features of **bedGraph files**
    
    .. attribute:: color
       Gradient of colors that assign by value to display in the genome browser
       
    Default fields are
        ::
        
         ['chr','start','end', 'score']
    
    :returns: BedGraph object  
          
    """

    def __init__(self, data, **kwargs):
        kwargs['format'] = 'bedGraph'
        kwargs['fields'] = ['chr','start','end','score']        
        self.color_gradient = kwargs.get('color',_blue_gradient)
#         GenomicContainer.__init__(self,data,**kwargs)
#         BedToolConvertible.__init__(self, **kwargs)
        BedToolConvertible.__init__(self,data, **kwargs)
        
    def win_mean (self):
        n_tracks = len (self.track.split("_"))

        # TODO if number of tracks is 1 exit returning self
        self.data = self._win_mean(self.data, n_tracks)
        return (self)
    
    def _win_mean (self, data, n_tracks):     
        for row in data:
            temp_list = []
            for i, v in enumerate(row):
                
                if i == 3:
                    temp_list.append (v/n_tracks)
                else:
                    
                    temp_list.append (v)
                
            yield (tuple(temp_list))


class Gff(BedToolConvertible):
    """
    A :class:`~pergola.tracks.GenomicContainer.BedToolConvertible` object designed 
    to include specific fields and features of **gff files**
    
    Default fields are
        ::
        
         ['seqname','source','feature','start','end','score',
          'strand','frame','attribute']
    
    :returns: Gff object    
        
    """
    def __init__(self, data, **kwargs):
        kwargs['format'] = 'gff'

        kwargs['fields'] = ['seqname','source','feature','start','end','score',
                            'strand','frame','attribute'] 
        
#         ['chr','chrom_start','chrom_end','name','score','strand',
#                             'thick_start','thick_end','item_rgb']

        BedToolConvertible.__init__(self,data,**kwargs)


def assign_color(set_data_types, color_restrictions=None):
    """
    Assign colors to fields randomly. It is optional to set given color to given 
    data_types,  a restricted color will no be used for the remaining data_types.
    
    :param set_data_types: :py:func:`set` containing data_types names that should 
        be linked to color gradient
    :param color_restrictions: A :py:func:`dict` that has as key data_types and as  
        values colors that are set by the user          
        ::  
        
            {'dataType': 'black'}
            
        Possible color gradients are
        ::
         
             'black', 'blue', 'red', 'green', 'orange'
    
    :returns: d_dataType_color dictionary with data_types as keys and color gradients
        as values
    
    """
    d_dataType_color = {}
    colors_not_used = []
    
    if color_restrictions is not None:
        rest_colors = (list (color_restrictions.values()))

        ## If there are restricted colors they should be on the default colors list
        if not all(colors in _dict_colors for colors in rest_colors):
            raise ValueError("Not all restricted colors are available") 
             
        ## If there are fields link to related colors they also must be in the data type list
#         if not all(key in set_data_types for key in color_restrictions):                                  
#             raise ValueError("Some values of data types provided as color restriction are not present in the file", (set_data_types, color_restrictions))
            
        for dataType in color_restrictions:
            d_dataType_color[dataType] = _dict_colors[color_restrictions[dataType]] 
    
        colors_not_used = list(__dict_colors.keys())
        colors_not_used.remove (color_restrictions[dataType])

    for dataType in set_data_types:        
        if not colors_not_used:
            colors_not_used = list(_dict_colors.keys())
        
        if dataType in d_dataType_color:
            continue
        else:
            d_dataType_color[dataType] = _dict_colors[colors_not_used.pop(0)]    
    
    return d_dataType_color


def merge_tracks (tr_1, tr_2):
    """
    Merge two tracks objects.    
    
    :param tr_1: Object of class :py:class:`~pergola.tracks.Track`  
    :param tr_2: Object of class :py:class:`~pergola.tracks.Track`  
       
    :returns:  dictionary with data_types as keys and color gradients
        as values
    """    
    merge_track = tr_1
    
    if set(tr_1.fields) != set(tr_2.fields):        
        raise ValueError ("Fields of the two tracks objects do not match" )
    
    if tr_1.format != tr_2.format:        
        raise ValueError ("Format of the two tracks objects do not match" )
    
    merge_track.data_types = tr_1.data_types.union (tr_2.data_types)
    merge_track.range_values = [min(tr_1.range_values[0], tr_2.range_values[0]), max(tr_1.range_values[1], tr_2.range_values[1])]
    
    merge_track.list_data_types = merge_track.data_types    
    
    merge_track.max = max (tr_1.max, tr_2.max)
    merge_track.min = min (tr_1.min, tr_2.min)
    
    merge_track.list_tracks = tr_1.list_tracks.union (tr_2.list_tracks)
    merge_track.data.extend (tr_2.data)                
        
    i_track_1 = tr_1.fields.index('track')
    i_start_1 = tr_1.fields.index('start')
    i_track_2 = tr_2.fields.index('track')
    i_start_2 = tr_2.fields.index('start')
    
    if i_start_1 != i_start_2:
        raise ValueError ("Start field does not match track object to append")
    
    if i_track_1 != i_track_2:
        raise ValueError ("Track field does not match track object to append")

    if all(row[i_track_1].isdigit() for row in merge_track.data):
        merge_track.data = sorted(merge_track.data, key=lambda x: (int(x[i_track_1]), x[i_start_1]))
    else:
        merge_track.data = sorted(merge_track.data, key=itemgetter(i_track_1, idx_fields2int))
                                                  
    return merge_track



#!/usr/bin/env python

from pergola import mapping
from pergola import intervals
from pergola import parsers

mapping_info = mapping.MappingInfo("/Users/jespinosa/git/pergola/sample_data/feeding_behavior/b2g.txt")
 
mapping_info.write()
 
# load the data into an IntData object that will store the sequence of events
int_data = intervals.IntData("/Users/jespinosa/git/pergola/sample_data/feeding_behavior/feedingBehavior_HF_mice.csv", map_dict=mapping_info.correspondence, delimiter="\t")
 
print (int_data.data[:12])
print (int_data.fieldsB)
print (int_data.fieldsG_dict)
# print (int_data.fieldsG_dict.keys())
print (int_data.dataTypes)
print (int_data.tracks)
print "debugging_simple_read---- min value in int_data_feeding ------------------------", int_data.min
print "debugging_simple_read---- max value in int_data_feeding ------------------------", int_data.max
track_data = int_data.read(relative_coord=True)
print "track object files are============+++++++++++++++++++", track_data.fields
track_data.convert(mode="bed")

#electro for single time point validation

mapping_info_e = mapping.MappingInfo("/Users/jespinosa/git/pergola/pergola/test/electrophysiology/e2p.txt")
mapping_info_e.write()

print ("debugging_simple_read correspondence.........................................................",mapping_info_e.correspondence.values())

int_data_e = intervals.IntData("/Users/jespinosa/git/pergola/pergola/test/electrophysiology/electroTest_2f.txt", map_dict=mapping_info_e.correspondence, delimiter="\t")
int_data_e_int = intervals.IntData("/Users/jespinosa/git/pergola/pergola/test/electrophysiology/electroTest_2f.txt", map_dict=mapping_info_e.correspondence, delimiter="\t")
  
print ("debugging_simple_read keys fieldsG_dict................",int_data_e.fieldsG_dict.keys())
print ("debugging_simple_read****************************************", int_data_e)
print ("debugging_simple_read", int_data_e.data[:12])
print ("debugging_simple_read",int_data_e.fieldsB)
print ("debugging_simple_read",int_data_e.fieldsG_dict)
print ("min in electrophysiology================================", int_data_e.min)
print ("min in electrophysiology================================", int_data_e.max)
# print (int_data.fieldsG_dict.keys())
print ("debugging_simple_read",int_data_e.dataTypes)
print ("debugging_simple_read",int_data_e.tracks)
# int_data_e.read(multiply_t=1000, relative_coord=True)# este tiene que petar no endChrom
print ("debugging_simple_read",int_data_e_int.data)
tracks_data_e = int_data_e_int.read(multiply_t=1000, intervals=True)
print "debugging_simple_read", tracks_data_e
print "debugging_simple_read", int_data_e_int.range_values
print "debugging_simple_read", tracks_data_e.range_values
print "debugging_simple_read", tracks_data_e.max
print "debugging_simple_read", tracks_data_e.data[0]







#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
=========================
Module: pergola.parsers
=========================

.. module:: parsers

This module provides the way to read ISA-tab format.


"""
from __future__ import print_function

from sys       import stderr
from bcbio     import isatab 
from os.path   import join, isfile, exists, isdir
from urllib2   import urlopen, HTTPError, URLError

def parse_isatab_assays(isatab_dir):
    """ 
    Read all files contained in isatab format to be processed by pergola
    
    :param isatab_dir: :py:func:`str` containing the path to isatab data folder
    
    :return: :py:func:`dict` of files to be processed by pergola
     
    TODO: This function needs that the assays to be processed are tagged some way
    
    """

    dict_files = dict()
    
#     if not path.isdir(isatab_dir):
    if not isdir(isatab_dir):
        raise ValueError ("Argument input must be a folder containning data in isatab format")
    
    rec = isatab.parse(isatab_dir) 
    
    # Sample name are the key shared by both study and assay
    for i in rec.studies:
        for j in i.assays:
            for file in j.nodes.keys():
                key = j.nodes[file].metadata['Sample Name'][0]

                dict_files[key] = file

    return dict_files

def check_assay_pointer(pointer, download_path):
    """
    Checks whether the argument pointer is the path to a local file or it is a URL
    If it is a URL it downloads the file to $HOME/.pergola if it has not been previously downloaded 
        
    :param pointer: :py:func:`str` path to a file or URL
    :param download_path: :py:func:`str` path to download files if they are specified as an URL  
    
    :returns: path of file to be processed 
           
    """

    # We check that the files has not been previously downloaded 
    file_name = pointer.split('/')[-1]
    path_file = join(download_path, file_name)
    
    # Checking if pointer is a file
    if isfile(pointer):
       print("\nPointer in isatab assays \"%s\" is a file in the system" % pointer, file=stderr)
       return (pointer)   
    elif exists(path_file):
        print("File has already been downloaded before: %s" % path_file, file=stderr)
        return (path_file)
    else:
        if not internet_on():  raise URLError("Check your network connection")
        
        try:
            url_file = urlopen(pointer)
            local_file = open(path_file, "w")
            local_file.write(url_file.read())
            print("\nFile %s has been correctly downloaded to %s"%(file_name, download_path))
            return path_file
        except (HTTPError, ValueError):
            raise ValueError("Pointer inside isatab assays table is either a file in your system nor a valid URL %s: " %
                             pointer)
        
def internet_on():
    """
    Checks whether there is an available internet connection
    
    :returns: :py:func:`boolean` True when internet connection is available otherwise False
    
    """

    try:
        response=urlopen('http://www.google.com', timeout=1)
        return True
    except URLError as err: pass
    return False



#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
=========================
Module: pergola.parsers
=========================

.. module:: parsers

This module provides the way to read scripts options provided by pergola library.


"""
from __future__ import print_function
from __future__ import absolute_import

from ._version  import __version__
from sys       import stderr
from argparse  import ArgumentParser, ArgumentTypeError
from re        import match
from os.path   import abspath, split, realpath
from .mapping   import check_path

_csv_file_ext = ".csv"

_dt_act_options = ['all', 'one_per_channel']
_tr_act_options = ['split_all', 'join_all', 'join_odd', 'join_even']
_starting_phase_options = ['light', 'dark']

PATH = abspath(split(realpath(__file__))[0])


def parse_num_range(string):
    """ 
    This function generate a numeric range from a string containing the boundaries
    From 1-4 generates a string  
    
    :param tracks: :py:func:`set` of tracks to which track_action should be applied set([1,2])
    :param delimiter: :py:func:`str` option to join tracks (join_all, split_all, join_odd, join_evens)
            
    :return: :py:func:`set` with all the numbers in range as strings
    
    """
    
    m = match(r'(\d+)(?:-(\d+))?$', string)

    if not m:
        raise ArgumentTypeError("'" + string + "' is not a range of number. Expected '0-5' or '2'.")
    start = m.group(1)
    end = m.group(2) or start
    list_range=list(range(int(start,10), int(end,10)+1))
    set_range=set(['{0}'.format(t) for t in list_range]) #str because track can be set in the form of track_1 for instance
    
    return set_range


def read_track_actions (tracks, track_action = "split_all"):
    """ 
    Read track actions and returns a set with the tracks to be joined
    
    :param tracks: :py:func:`set` of tracks to which track_action should be applied set([1,2])
    :param track_action: :py:func:`str` option to join tracks (join_all, split_all, join_odd, join_evens)
    
    :return: :py:func:`set` of tracks to be joined
     
    """
    
    if track_action not in _tr_act_options:
        raise ValueError("Track_action \'%s\' not allowed. Possible values are %s"%(track_action,', '.join(['{}'.format(m) for m in _tr_act_options])))
    
    tracks2merge = ""
    
    if track_action == "join_all":
        tracks2merge = tracks
    elif track_action == 'join_odd':
        tracks2merge = set([t for t in tracks if int(t) % 2])
    elif track_action == 'join_even':
        tracks2merge = set([t for t in tracks if not int(t) % 2])
    else:
        tracks2merge = ""
        
    print("Tracks to merge are: ", ",".join("'{0}'".format(t) for t in tracks2merge), file=stderr)
       
    if not tracks2merge:
        print(("No track action applied as track actions \'%s\' can not be applied to list of tracks provided \'%s\'"%(track_action, " ".join(tracks))), file=stderr)
        
    return (tracks2merge)


def read_colors (path_color_file):
    """     
    Reads user colors for each data_type  
    
    :param None path_color_file: :py:func:`str` path to read user color for data_types     
       
    :returns: d_user_color dictionary {'data_type_1': 'orange', 'data_type_2':'blue'}
    
    """

    check_path(path_color_file)
    comment_tag_t = "#"    
    d_user_color = {}                            
    
    with open(path_color_file) as f:    
       
       for row in f:
           
           if(row.startswith(comment_tag_t) or row.startswith('\n')):               
                continue
                       
           row_split = row.rstrip('\n').split('\t') 
           (data_type, color) = row_split
           
           # colors are checked inside tracks.assign_color
           d_user_color[data_type] = color
    
    return d_user_color

""""   
Parsers arguments of pergola_rules.py script (aka pergola)

"""

parent_parser = ArgumentParser(description = 'Script to transform behavioral data into GB readable data', add_help=False)

parent_parser.add_argument('-i', '--input', required=True, metavar="PATH", nargs='+', help='Input file path')
parent_parser.add_argument('-m', '--mapping_file', required=True, metavar="MAPPING_FILE",
                    help='File to set the reciprocity between fields in behavioral file and terms used by Pergola' + \
                    ' and genome browser grammar')
parent_parser.add_argument('-t', '--tracks', required=False, metavar="TRACKS", type=int, nargs='+', 
                    help='List of selected tracks')
parent_parser.add_argument('-l','--list', required=False, metavar="LIST_OF_TRACKS", type=str, nargs='+',
                           help='Numeric list of tracks to be joined in a single genomic like file')### string allowed as some tracks could be named as: track_1, track2....
parent_parser.add_argument('-r', '--range', required=False, type=parse_num_range,
                           help='Numeric range of tracks to be joined in a single genomic like file')
parent_parser.add_argument('-a', '-ta', '--track_actions', required=False, choices=_tr_act_options,
                           help='Option of action with tracks selected, split_all, join_all,' + \
                           ' join_odd, join_even, join_range or join_list')
parent_parser.add_argument('-dl', '--data_types_list', required=False, metavar="LIST_OF_DATA_TYPES", type=str, nargs='+',
                           help='List of data_types to be joined')
parent_parser.add_argument('-d', '--data_types_actions', required=False, choices=_dt_act_options,
                           help='Unique values of data_types field should be dumped on' + \
                           ' different data structures or not')
parent_parser.add_argument('-f', '--format', required=False, type=str, default='bed',
                           help='Write file output format (bed or bedGraph)')
parent_parser.add_argument('-e', '-rel', '--relative_coord', required=False, action='store_true',
                           default=False, help='Sets first timepoint' \
                           ' to 0 and make all the others relative to this timepoint')
parent_parser.add_argument('-n', '-int', '--intervals_gen', required=False, action='store_true',
                           default=None, help='Set startChrom and endChrom from just a timepoint in the file ' \
                           'using field set as startChrom')
parent_parser.add_argument('-ns', '-int_s', '--interval_step', required=False, metavar="INTERVAL_STEP", type=int,
                           default=False, help='Set step to generate intervals from a single time point')
parent_parser.add_argument('-mi', '--multiply_intervals', metavar='N', type=int, required=False,
                           help='Multiplies value in data_value by the given factor')
parent_parser.add_argument('-nh', '--no_header', required=False, action='store_true', 
                           default=False, help='Data file contains no header')
parent_parser.add_argument('-s', '-fr', '--fields_read', metavar='FIELDS2READ', type=str, nargs='+',
                           help='List of fields to read from input file')
parent_parser.add_argument('-w', '--window_size', required=False, metavar="WINDOW_SIZE", type=int, 
                           help='Window size for bedGraph intervals, default value 300')
parent_parser.add_argument('-nt', '--no_track_line', required=False, action='store_true',
                           default=False, help='Track line no included in the bed file')
parent_parser.add_argument('-fs', '--field_separator', required=False, type=str,
                           default=False, help='Input file field separator')
parent_parser.add_argument('-bl', '--bed_label', required=False, action='store_true',
                           default=False, help='Show data_types as name field in bed file')
parent_parser.add_argument('-c', '--color_file', required=False, metavar="PATH_COLOR_FILE", 
                           help='Dictionary assigning colors of data_types path')
parent_parser.add_argument('-wm', '--window_mean', required=False, action='store_true',
                           default=False, help='Window values averaged by the window size')
parent_parser.add_argument('-vm', '--value_mean', required=False, action='store_true',
                           help='Window values averaged by number of items within window')
parent_parser.add_argument('-min', '--min_time', type=int, required=False,
                           help='Initial time point to extract')
parent_parser.add_argument('-max', '--max_time', type=int, required=False,
                           help='Last time point to extract')
parent_parser.add_argument('-ng', '--no_genome', required=False, dest='genome', action='store_false',
                           help='Avoinds the creation of a FASTA file which allows to render a longitudinal trajectory' \
                           ' in a genome browser')
parent_parser.add_argument('-np', '--no_phases', required=False, dest='phases', action='store_false',
                           help='Avoids the creation of a phases bed file')
parent_parser.set_defaults(genome=True)
parent_parser.set_defaults(phases=True)
parent_parser.add_argument('-o', '--output_file_name', help='File name for output files')
parent_parser.add_argument('-sp', '--starting_phase', type=str, required=False, choices=_starting_phase_options,
                           help='Sets the first phase to appear in the phases and cytoband file: light or dark\n')
parent_parser.add_argument('-sh', '--shift', required=False, metavar="TIME_SHIFT", type=int,
                           help='Time shift to be set for the first phase')
parent_parser.add_argument('-v', '--version', action='version', version='%(prog)s {version}'.format(version=__version__))

""""   
Parsers argument of jaaba_to_pergola.py script

"""

jaaba_parser = ArgumentParser(description = 'Script to transform Jaaba annotations into Pergola readable formats', 
                              add_help=False)
jaaba_parser.add_argument('-v', '--version', action='version', version='%(prog)s {version}'.format(version=__version__))

subparsers = jaaba_parser.add_subparsers(help='Calls pergola_rules.py', dest='command')
jaaba_parser_sp = subparsers.add_parser('sp', help="Converts Jaaba data and process it using pergola", parents=[parent_parser])
jaaba_parser_sc = subparsers.add_parser('sc', add_help='Converts scores Jaaba files into csv files')
jaaba_parser_fp = subparsers.add_parser('fp', add_help='Converts Jaaba features using pergola', parents=[parent_parser])
jaaba_parser_fc = subparsers.add_parser('fc', add_help='Converts Jaaba features into csv files')

jaaba_parser_sc.add_argument('-i', '--input', required=True, metavar="PATH", help='Input file path')

jaaba_parser_fc.add_argument('-i', '--input', required=True, metavar="PATH", help='Path to jaaba features')
jaaba_parser_fc.add_argument('-jf', '--feature', required=True, metavar="LIST_OF_FEATURES", type=str, nargs='+',
                            help='List of features to be extracted, e.g. velmag')
jaaba_parser_fc.add_argument('-dd', '--dumping_dir', required=False, metavar="DUMPING_DIR", help='Input file path')
# jaaba_parser_fp.add_argument('-i', '--input_jaaba_dir', required=True, metavar="PATH", help='Input file path')
# jaaba_parser_fp.add_argument('-jd', '--jaaba_dir', required=True, metavar="PATH", help='Path to jaaba features')
jaaba_parser_fp.add_argument('-jf', '--feature', required=True, metavar="LIST_OF_FEATURES", type=str, nargs='+',
                            help='List of features to be extracted, e.g. velmag')
jaaba_parser_fp.add_argument('-dd', '--dumping_dir', required=False, metavar="DUMPING_DIR", help='Input file path')



#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
=========================
Module: pergola.intervals
=========================

.. module:: intervals

This module provides the way to read the input intervals files.
It contains a class :class:`~pergola.intervals.IntData` which has 
the attributes and methods needed for reading the data.

"""
from __future__ import print_function
from __future__ import absolute_import

from .mapping  import check_path
from csv      import reader
from sys      import stderr
from .tracks   import Track
from operator import itemgetter
from pandas   import read_excel

class IntData(object):
    """
    Generic class for input data
    
    .. attribute:: path
    
       Name of path to a csv/tab or xlsx (excel) input file
    
    .. attribute:: delimiter
    
       Character use to separate values of the same record in file (default "\t").
    
    .. attribute:: header
    
       Indicates the presence of a header.
       * `False` if there is no header. Fields should the be provided using fields param
       * `True` if the file have a header line with names. This names should match names in map_dict (default).
    
    .. attribute:: fieldsB
    
        List with the behavioral fields corresponding each column in the file
    
    .. attribute:: fieldsG_dict
    
        A dictionary with the equivalence to map fields in the input file and the pergola
        ontology to describe behavioral data    
    
    .. attribute:: min
    
        First time value in the file. Read from field set as "start"
    
    .. attribute:: max
    
        Last timepoint in the file. Read from field set as "end"
    
    .. attribute:: range_values
    
        Range of values inside data_value field
        
    .. attribute:: data
    
        List of tuples containing the data read from the file
    
    .. attribute:: data_types
    
        All different data_types that appear in the data read from "data_types" field.
        If data_types field not in file, all intervals are set as belonging to data_types "a"

    .. attribute:: tracks
    
        Set of tracks in the file. Read from "tracks" field.
        If tracks field not in file, all intervals are set as belonging to track "1" 
        
    :returns: IntData object
    
    """

    def __init__(self, path, map_dict, header=True, **kwargs):
        self.path = check_path(path)
        self._in_file = open(self.path, "r")
        self.delimiter = self._check_delimiter(self.path, kwargs.get('delimiter', "\t"))
        self.header = header
        self._reader = self._reader_data()
        self.fieldsB = self._set_fields_b(kwargs.get('fields_names', None))
        self.fieldsG_dict = self._set_fields_g(map_dict)
        self.fieldsG = list(self.fieldsG_dict.keys()) #here before I added the new fields

        self.min = self.max = 0
        self.range_values = 0
        self.data = self._simple_read()
#         self.data = self._read(multiply_t = kwargs.get('multiply_t', 1), intervals=kwargs.get('intervals', False))
        self.data_types = self.get_field_items(field ="data_types", data = self.data, default="a")
        self.tracks = self.get_field_items(field="track", data = self.data, default="1")#TODO maybe this function will be more general if instead of giving field name
        #i pass the index

    def _check_delimiter (self, path, delimiter):
        """ 
        Check whether the set delimiter works, if delimiter not set then tries ' ', '\t' and ';'
         
        :param path: :py:func:`str` path to a behavioral file in the form of a csv file
        :param delimiter: :py:func:`str` delimiter used in the file ("tab", ";", "space") 
        
        :returns: delimiter
        
        """
        ext = self.path.rpartition('.')[-1].lower()
            
        if ext == "xlsx":
            return "\t"

        for row in self._in_file:            

            # Comments skipped
            if row.startswith("#"):
                continue

            # Delimiter set by user
            if row.count(delimiter) >= 1: break
            else: raise ValueError("Input delimiter does not correspond to delimiter found in file \'%s\'"%(delimiter))

            # Delimiter guess by function
            if row.count(" ") >= 1:
                self.delimiter = " "
                break
            if row.count("\t") >= 1:
                self.delimiter = "\t"
                break
            if row.count(";") >= 1:
                self.delimiter = ";"
                break

        if delimiter is None:
            raise ValueError("Delimiter must be set \'%s\'"%(delimiter))

        self._in_file.seek(0)

        return delimiter

    def _reader_data(self):
        """
        Reads input file into a generator depending on its formats

        :returns: :py:func:`generator` containing data from input file

        """
        ext = self.path.rpartition('.')[-1].lower()

        if ext == "csv":
            print(("Input file format identified as csv"), file=stderr)
            reader_obj = reader(self._in_file, delimiter=self.delimiter)
            return reader_obj
        elif ext == "xlsx":
            print(("Input file format identified as xlsx"), file=stderr)
            reader_obj = self._pandas_df_reader(read_excel(self.path, header=None, sheet_name=0, index=False))
            return reader_obj
        else:
            print(("WARNING: File format not recognized, default format assumed to be csv"), file=stderr)
            reader_obj = reader(self._in_file, delimiter=self.delimiter)
            return reader_obj

    def _pandas_df_reader(self, df):
        """
        Reads pandas dataframe from excel file and returns generator

        :param df: pandas :py:func:`dataframe`

        :returns: :py:func:`generator` from excel file

        """

        for r in df.itertuples():
            yield [str(i) for i in list(r[1:])]

    def _set_fields_b(self, fields=None):
        """
        Reading the behavioral fields from the header file or otherwise setting  
        the fields to numeric values corresponding the column index starting at 0
        
        :param None fields: :py:func:`list` with the behavioral fields corresponding to each column in the file
        
        :returns: list with the behavioral fields
            
        """

        fieldsB = []
        first_l = []

        for row in self._reader:
            if row[0].startswith("#"):
                continue
            else:
                first_l = row
                break

        if self.header:
            header = first_l
            first_r = next(self._reader)

            if len(header) != len(first_r):
                raise ValueError("Number of fields in header '%d' does not match number of fields in first row '%d'"
                                 % (len(header), len(first_r)))

            if fields:
                if len(fields) > len(first_r):
                    raise ValueError("Input field list \"%s\" is longer than totals fields available in file \'%s\'"
                                     % ("\",\"".join(fields), len(first_r)))

                if not all(field in header for field in fields):
                    raise ValueError("Input field list \"%s\" has items not present in file header \'%s " 
                                     '\n'
                                     "Also make sure you don't need to set header=False"
                                     % ("\",\"".join(fields), "\",\"".join(header)))

                ori_fieldsB = [header[0].strip('# ')]+header[1:]

                for f in ori_fieldsB:
                    if f in fields: fieldsB.append(f)
                    else: fieldsB.append("")

            else:
                fieldsB = [header[0].strip('# ')]+header[1:]
        else:
            first_r = first_l

            if fields and fields[0] != "ordinal":
                if len(fields) > len(first_r):
                    raise ValueError("Input field list \"%s\" is longer than totals fields available in file \'%s\'" % ("\",\"".join(fields), len(first_r)))

                fieldsB = fields

                print(("WARNING: As header=False you col names set by fields will be considered to have the order "
                        "you provided: \"%s\""
                        %"\",\"".join(fields)), file=stderr)
            elif fields and fields[0] == "ordinal":
                fieldsB = range (1, len(first_r)+1)
                fieldsB = [str(i) for i in fieldsB]
            else:
                raise ValueError ('File should have a header, otherwise you should set ' 
                                  'an ordered list of columns names using fields')

#         self.in_file.close()
        self._in_file.seek(0)

        return fieldsB

    def _set_fields_g (self, map_dict):
        """
        Extracts the correspondence of fields in genomic grammar of the behavioral file.
        
        :param map_dict: relationship between behavioral data fields and pergola ontology (:py:class:`dict`)
        
        :returns: list with the corresponding genomics names of the fields inside behavioral (input) data
         
        """

        dict_fields_g = {}
        i_field_b=0

        if len(self.fieldsB) != len(map_dict):
            print(("WARNING: Number of fields in input file (%d) does not match number of fields in "
                              "mapping file (%d)." % (len(self.fieldsB), len(map_dict))), file=stderr)

        for field_B in self.fieldsB:
            if field_B:
                try:
                    map_dict [field_B]
                except KeyError:
                    raise KeyError ("Field %s is not mapped in your ontology mapping. " \
                                    "TIP: Fields that are not use from the input data have to be set to dummy " \
                                    "in the ontology mapping. Example: behavioral_file:%s > pergola:dummy"
                                    % (field_B, field_B))
                dict_fields_g[map_dict [field_B]] = i_field_b
            i_field_b = i_field_b + 1

        name_fields_g = [map_dict[k] for k in self.fieldsB if k]

        #Input file at least should have two fields that correspond to:
        mandatory_fields = ["start", "data_value"]

        if not all(f in dict_fields_g.keys() for f in mandatory_fields):
            raise ValueError("Input file mandatory fields  are \"start\" and \"data_value\" \n" \
                             "Your current assigned fields are \"%s\"\n" \
                             "TIP: Check your ontology_file"
                             % ("\",\"".join(name_fields_g)))

        return dict_fields_g

    def _simple_read(self):
        """
        This function just needs to read the raw data set min and maximum, data_types and this stuff
        _read was too complicated
        
        :returns: list with intervals contained in file, minimum and maximum values inside the file
         
        """

        list_data = list()
        header_check = False
#
        for row in self._reader:
            # Comments skipped
            if row[0].startswith("#"):
                continue

            if self.header and not header_check:
                header_check = True
                continue

            if isinstance((row[self.fieldsG_dict["start"]]), str):
                row[self.fieldsG_dict["start"]] = num(row[self.fieldsG_dict["start"]])

            if "end" in self.fieldsG_dict and isinstance((row[self.fieldsG_dict["end"]]), str):
                    row[self.fieldsG_dict["end"]] = num(row[self.fieldsG_dict["end"]])

            list_data.append(tuple(row)) #TODO what is better tuple or list

        # Initialize min, max
        self.min, self.max = self._min_max(list_data)

        # Initialize range_values
        self.range_values = list(self._min_max(list_data, f_start="data_value", f_end="data_value"))

        # Back to file beginning
        self._in_file.seek(0)

        return (list_data)

    def get_field_items(self, data, field="data_types", default=None):
        """
        Reads the unique values inside a field and returns them as a set
        If default is set and field does not exist in the data then the field is
        added to the the data and set to default value
        
        :param data: :py:func:`list` with the intervals read from data 
        :param field: :py:func:`str` field from data from which to inferred set of 
            unique values
        :param None default: if field is not present in data is created and set to 
            default
            
        :returns: set with unique values inside field
        
        """

        set_fields = set()

        if field in self.fieldsG:
            idx_field = self.fieldsG_dict[field]
            field = [field]

            for row in self.data:
                set_fields.add(row[idx_field])

        elif default:
            new_data = list()
            new_field = (default,)

            set_fields.add(default)

            for row in self.data:
                row = row + new_field
                new_data.append(row)

            self.data = new_data
            pos = len(self.fieldsG)
            self.fieldsG.append(str(field))
            self.fieldsG_dict[field]=pos

        else:
            raise ValueError("Data has not field \'%s\' and no default value has been set \'%s\'"%(field, default))

        return set_fields

    def read(self, fields=None, relative_coord=False, intervals=False, int_step=None, fields2rel=None, multiply_t=None,
             **kwargs):
        """        
        Reads the data and converts it depending on selected options
        
        :param None fields: :py:func:`list` with data columns to read
        :param False relative_coord: If true all coordinates in start and end are
            make relative to the minimal value
        :param False intervals: if set to true intervals will be inferred from timepoints in
            start
        :param None int_step: :py:func:`int` :param None int_step: :py:func:`int` time step value to create 
            the end of intervals
        :param None fields2rel: :py:func:`list` with data columns to make relative
        :param None multiply_t: :py:func:`int` multiplies the values of the field set as start and 
            end        
        
        :returns: Track object
        
        TODO: By the moment I make this function as a method of the class eventually I would make this a
        separated function
        Eventually do not change self.data but a list inside read and return the Track object with the modifications
        this way data is always the original one.
        
        """

        _f_rel_mand = "start"
        _f_int_end = "end"
        _f2rel = ["start","end"]
        _f2mult = ["start","end"]
        i_time_f = [10000000000000]

        # If fields is not set then all the data columns are read
        if fields is None:
            fields = self.fieldsG
            indexL = range(len(self.fieldsG))
        else:
            try:
                indexL = [self.fieldsG.index(f) for f in fields]

            except ValueError:
                raise ValueError("Field '%s' not in file %s." % (f, self.path))

        # If start not present out     
        try:
            idx_fields2int = self.fieldsG_dict[_f_rel_mand]
        except ValueError:
            raise ValueError("Parameter intervals=True needs that field '%s' is not missing in file %s."
                             % (_f_rel_mand, self.path))

        ##################################
        # If there are several tracks we order by track
        # Control for interval change bw tracks        
        _f_track = "track"
        i_track = None

        if _f_track in self.fieldsG_dict:
            i_track = self.fieldsG_dict[_f_track]

            if all(row[i_track].isdigit() for row in self.data):
                self.data = sorted(self.data, key=lambda x: (int(x[i_track]), x[idx_fields2int]))
            else:
                self.data = sorted(self.data, key=itemgetter(i_track, idx_fields2int))

        # Coordinates multiplied by a given factor set by the user
        if multiply_t:
            print("Fields containing time points will be multiplied by: ", multiply_t, file=stderr)

            try:
                f=""
                name_fields2mult = [f for f in _f2mult if f in self.fieldsG_dict]
                idx_fields2mult = [self.fieldsG_dict[f] for f in name_fields2mult]
            except ValueError:
                raise ValueError("Field '%s' not in file %s." % (f, self.path))

            self.data = self._multiply_values(i_fields=idx_fields2mult, factor=multiply_t)

        # Coordinates transformed into relative to the minimun time point
        print("Relative coordinates set to:", relative_coord, file=stderr)

        if relative_coord:
            if fields2rel is None:
                # Do I have intervals or single points
                f2rel = list(set(_f2rel) & set(self.fieldsG))
                if f2rel is None:
                    raise ValueError("You need at least a field containing time points when relative_coord=T. %s" % (self.fieldsG))

            else:
                # Are the provided fields present in data and are numeric #TODO en realidad si no es numerico ya petara
                if isinstance(fields2rel, str): fields2rel = [fields2rel]
                f2rel = [f for f in fields2rel if f in self.fieldsG]

            # Getting indexes of fields to relativize
            try:
                i_time_f = [self.fieldsG_dict[f] for f in f2rel]
            except ValueError:
                raise ValueError("Field '%s' not in file %s mandatory when option relative_coord=T." % (f, self.path))

            self.data = self._time2rel_time(i_time_f)

        # From only start value for each time point we generate intervals
        if intervals:
            print("Intervals will be inferred from timepoints", file=stderr)

            if _f_int_end in self.fieldsG_dict:
                raise ValueError("Intervals can not be generated as '%s' already exists in file %s." % (_f_int_end, self.path))

            self.data = self._create_int(idx_fields2int, int_step)
            # self.data = self._create_int_add_integ(idx_fields2int)

        # If min and max set by argument take them after converting to relative coordinates
        # set by max_time or min_time argument #TODO
        # self.min = kwargs.get('min_time', self.min)
        # self.max = kwargs.get('max_time', self.max)
        min_t = kwargs.get('min_time', self.min)
        max_t = kwargs.get('max_time', self.max)

        if min_t >= max_t: raise ValueError("FATAL ERROR: Minimum time point \'%i\' can not be greater than maximum " \
                                            "time point \'%i\'" % (min_t, max_t))

        self.min = min_t
        self.max = max_t

        if intervals:
            self.max = self.max + 1

        # To continue intervals are mandatory
        try:
            i_max = self.fieldsG_dict[_f_int_end]
        except KeyError:
            raise KeyError("Field '%s' for max interval calculation time not in file %s. " \
                           "TIP: You can transform timepoints to intervals setting intervals=True"
                           % (_f_int_end, self.path))

        # Updated and order list of the fields        
        list_fields = [None] * len(self.fieldsG_dict)

        for field, i in self.fieldsG_dict.items():
            list_fields[i] = field

        self.fieldsG = list_fields

        return Track(self.data, self.fieldsG, data_types=self.data_types, list_tracks=self.tracks, range_values=self.range_values, min=self.min, max=self.max)

    def _min_max(self, list_data, f_start="start", f_end="end"):
        """
        Obtains minimum and maximum values from fields set by function parameters 
        
        :param list_data: :py:func:`list` with data columns from which to extract the minimum and
            maximum value
        :param f_start: :py:func:`str`           
        :param "start" f_start: :py:func:`str` Field from the list_data to extract minimum value        
        :param "end" f_end: :py:func:`str` Field from the list_data to extract minimum value
                
        :returns: minimum and maximum values
        
        """

        # Min and maximun time points
        t_min = None
        t_max = None

        i_time = self.fieldsG_dict[f_start]

        t_min = float(min(list_data, key=itemgetter(i_time))[i_time])

        if f_end in self.fieldsG_dict.keys():
            i_time = self.fieldsG_dict[f_end]

        line_max = max(list_data, key=lambda line: float(line[i_time]))
        t_max = float((line_max[i_time]))

        if t_min.is_integer():
            t_min = int(t_min)
        else:
            t_min = t_min

        if t_max.is_integer():
            t_max = int(t_max)
        else:
            t_max = t_max

        return t_min, t_max

    def _time2rel_time(self, i_fields):
        """
        Calculates relative values of selected data columns 
        
        :param i_fields: :py:func:`list` with data columns to calculate relative values
        
        :returns: list of tuples (self.data-like)
        
        TODO check whether field for min and max is the same as the one selected by i_fields otherwise
        give either exception of warning
        I have two problems with this, first that if i have intervals min will be only in one of the two
        fields set for convert into relative.
        The second problem is that the min is read far before here is where I have to check this
        In principal this should be always in start that is why I have the terms in the ontology!!!!!
        
        """

        data_rel = list()

        for row in self.data:
            temp = []
            for i in range(len(row)):

                if i in i_fields:

                    if is_number(row[i]):
                        n = float(row[i])

                        if n % 1 == 0:
                            # temp.append(int(row[i])- self.min + 1)
                            temp.append(int(row[i]) - self.min)
                        else:
                            raise ValueError("Value can not be relativize because is not an integer \'%.16f\'" \
                                            ". Use option -mi,--multiply_intervals n"%(row[i]))  #correct this is only true for pergola_rules
                else:
                    temp.append(row[i])

            data_rel.append((tuple(temp)))

        self.min, self.max = self._min_max(data_rel)

        return (data_rel)

    def _multiply_values(self, i_fields, factor=1):
        """
        Multiplicate values of selected data columns by the given factor
        
        :param i_fields: :py:func:`list` with data columns to calculate relative values
        
        :param factor: :py:func:`int` factor to multiply data columns selected
        
        :returns: list of tuples (self.data-like)
        
        TODO change min and max of the data to new values
        The simpler way is multiply this values as well
        I don't need to generate a temporal list do I?
        For this think maybe is better to have a list of list than a list of tuple
        
        """
        data_mult = list()

        for row in self.data:
            temp = []
            for i in range(len(row)):

                if i in i_fields:
                    value = row[i]
#                     value = row[i].replace(" ", "")

                    if is_number(value):
                        v_m = round (float(row[i]) * factor, 6)
                        v_i = int(v_m)
                        if v_m-v_i != 0:
                            raise ValueError ("Intervals values (start and end) can not be decimal\nPlease use a bigger factor " \
                                              "with -m,--multiply_intervals flag to multiply your values, current value is %s"%factor)
                        temp.append(v_i)

                    else:
                        raise ValueError("Value can not be multiplied because is not a number \'%s\'" \
                                            "\nCheck mapping of fields in your input file n"%(row[i]))  #corregir
                else:
                    temp.append(row[i])

            data_mult.append((tuple(temp)))

        self.min, self.max = self._min_max(data_mult)

        return (data_mult) # Correct eventually self.data in a list of list directly modificable

    def _create_int(self, start_int, int_step=None):
        """
        From single time points generates intervals of time

        :param start_int: :py:func:`int` with index containing time points
        
        :param None int_step: :py:func:`int` time step value to create the end of intervals
        
        :returns: list of tuples (self.data-like)
        
        """
        data_int = list()
        _f_int_end = "end"

        # Field is add as supplementary column
        end_int = len(self.fieldsG)
        self.fieldsG_dict[_f_int_end] = end_int
        self.fieldsG.append(_f_int_end)

        # I have to check whether track field exists
        # if exists then bw tracks I have a last row
        _f_track = "track"
        i_track = None
        track_sw = False

        if _f_track in self.fieldsG_dict:
            i_track = self.fieldsG_dict[_f_track]
            track_sw = True

        # All items except last
        for i in range(len(self.data[:-1])):
            # if p_track and p_track == self.data[i][i_track]:

            row = self.data[i]

            if int_step:
                value_end = (self.data[i][start_int] + int_step,)

            else:
                # If the track is still the same
                if track_sw:
                    # same track
                    if row[i_track] == self.data[i + 1][i_track]:
                        # if the following interval starts with the same start point then we can not use it
                        # and we just add one to the start of the interval
                        if (self.data[i][start_int],) == (self.data[i + 1][start_int],):
                            value_end = (self.data[i + 1][start_int] + 1,)
                        # otherwise we substract one to the next start point
                        else:
                            value_end = (self.data[i + 1][start_int] - 1,)
                    # different track
                    # we add one to the start point
                    else:
                        value_end = (row[start_int] + 1,)

                else:
                    if (self.data[i][start_int],) == (self.data[i + 1][start_int],):
                        value_end = (self.data[i + 1][start_int] + 1,)

                    else:
                        value_end = (self.data[i + 1][start_int] - 1,)

            temp = row + value_end
            data_int.append((tuple(temp)))

        # Last item
        last_row = self.data[-1]

        if int_step:
            value_end = (last_row[start_int] + int_step,)
        else:
            value_end = (last_row[start_int] + 1,)


        self.max = value_end[0]

        temp = last_row + value_end

        data_int.append((tuple(temp)))

        return (data_int)

    def _create_int_add_integ(self, start_int, integer=1):
        """
        From single time points generates intervals of time
        
        :param start_int: :py:func:`int` with index containing time points 
        
        :returns: list of tuples (self.data-like)
        
        """
        data_int = list()
        _f_int_end = "end"

        # Field is add as supplementary column
        end_int = len(self.fieldsG)
        self.fieldsG_dict[_f_int_end] = end_int
        self.fieldsG.append(_f_int_end)

        # All items except last
        for i in range(len(self.data)):

            row = self.data[i]
            value_end = (row[start_int] + integer,)

            temp = row + value_end
            data_int.append((tuple(temp)))

        return (data_int)

    def __del__(self):
        self._in_file.close()

def is_number(var):
    """
    Checks whether an string is a number, if is already an integer or float it also returns True
    
    :param factor: :py:func:`str` :py:func:`int` :py:func:`float` variable to check
    
    :returns: :py:func:`boolean` True when input variable is a number otherwise False
    
    """

    try:
        float(var)
        return True
    except ValueError:
        return False


def num(s):
    """
    Returns integer or float from string

    :param s: :py:func:`str` string to be transformed to numeric

    :returns: numeric type
    
    """

    try:
        return int(s)
    except ValueError:
        return float(s)




#!/usr/bin/env python
#
#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
30 oct 2014

Script to run pergola from the command line
"""
from __future__ import print_function

from pergola  import intervals
from pergola  import mapping
# from pergola  import tracks
from argparse import ArgumentParser
from sys      import stderr, exit
# import os
from os.path import basename, splitext
from pergola import parsers

def main(args=None):
    parser_pergola_rules = ArgumentParser(parents=[parsers.parent_parser]) 
    
    args = parser_pergola_rules.parse_args()

    multiple_files = False

    if len(args.input) > 1:
        print("@@@Pergola_rules.py: Multiple input files processed")
        multiple_files = True

    for idx, input_file in enumerate(args.input):

        if multiple_files:
            if args.output_file_name is None:
                output_file_n = splitext(basename(input_file))[0]
            else:
                output_file_n = args.output_file_name + "_" + str(idx + 1)
        else:
            output_file_n = args.output_file_name

        pergola_rules(path=input_file, map_file_path=args.mapping_file, sel_tracks=args.tracks,
                      list=args.list, range=args.range, track_actions=args.track_actions,
                      data_types_actions=args.data_types_actions, data_types_list=args.data_types_list,
                      write_format=args.format, relative_coord=args.relative_coord,
                      intervals_gen=args.intervals_gen, interval_step=args.interval_step,
                      multiply_f=args.multiply_intervals, no_header=args.no_header, fields2read=args.fields_read,
                      window_size=args.window_size, no_track_line=args.no_track_line, separator=args.field_separator,
                      bed_lab_sw=args.bed_label, color_dict=args.color_file, window_mean=args.window_mean,
                      value_mean=args.value_mean, min_t=args.min_time, max_t=args.max_time, phases=args.phases,
                      genome=args.genome, output_file_name=output_file_n, starting_phase=args.starting_phase,
                      shift=args.shift)

def pergola_rules(path, map_file_path, sel_tracks=None, list=None, range=None, track_actions=None, 
                  data_types_actions=None, data_types_list=None, write_format=None, relative_coord=False,
                  intervals_gen=False, multiply_f=None, no_header=False, fields2read=None, window_size=None,
                  no_track_line=False, separator=None, bed_lab_sw=False, color_dict=None, window_mean=False,
                  value_mean=False, min_t=None, max_t=None, interval_step=None, phases=False, genome=False,
                  output_file_name=None, starting_phase=False, shift=None):
    
    print("@@@Pergola_rules.py: Input file: %s" % path, file=stderr) 
    print("@@@Pergola_rules.py: Configuration file: %s" % map_file_path, file=stderr)
    
    # Tracks selected by user
    print("@@@Pergola_rules.py: Selected tracks are: ", sel_tracks, file=stderr)
    
    # Configuration file
    map_file_dict = mapping.MappingInfo(map_file_path)
    
    # Reading color dictionary to set data_types
    if color_dict:
        print("@@@Pergola_rules.py: Color for data_types in file............ %s" % color_dict, file=stderr)
        d_colors_data_types = parsers.read_colors (color_dict)
    else:
        d_colors_data_types = None    
        
    # Handling list or range of tracks to join if set
    if list and range:
        raise ValueError("@@@Pergola_rules.py: Argument -l/--list and -r/--range are not compatible. " \
                         "As both arguments set tracks to be joined.")    
    elif (list):
        tracks2merge = list
    elif (range):
        tracks2merge = range
    else:
        tracks2merge = ""
    
    if tracks2merge: print("@@@Pergola_rules.py: Tracks to join are............ ", tracks2merge, file=stderr)
    
    # Handling argument track actions
    if tracks2merge and track_actions:
        raise ValueError ("Options --list -l or --range -r are incompatible with " \
                          "--track_actions -a, please change your options")
    
    track_act = track_actions
    print("@@@Pergola_rules.py: Track actions are.............................. ", track_act, file=stderr)
    
    data_types_list = data_types_list
    print("@@@Pergola_rules.py: data_types list is: ", data_types_list, file=stderr)
      
    # Handling argument data_types actions
    data_types_act = data_types_actions
    print("@@@Pergola_rules.py: data_types actions are......................... ", data_types_act, file=stderr)
    
    # Handling argument format    
    if write_format:
        print("@@@Pergola_rules.py format to write files....................... ", write_format, file=stderr)
    else:
        write_format = 'bed'
        print("@@@Pergola_rules.py format to write files has been set" \
                        " to default value:", write_format, file=stderr)

    # Handling relative coordinates
    print("@@@Pergola_rules.py: Relative coordinates set to................. %s" % relative_coord, file=stderr)
    
    # Handling intervals_gen
    print("@@@Pergola_rules.py: Intervals parameter set to.................. %s" % intervals_gen, file=stderr)

    # Handling interval_step
    if interval_step:
        if intervals_gen:
            print("@@@Pergola_rules.py: Interval step set to........................ %s" % interval_step, file=stderr)
        else:
            raise ValueError("Interval step needs intervals paramater to be set -n/--intervals_gen")

    # Handling multiply_intervals
    if multiply_f:
        print("@@@Pergola_rules.py: Multiply intervals parameter set to...... %s" % multiply_f, file=stderr)                        
    else:
        multiply_f = 1
    
    print("@@@Pergola_rules.py: Selected tracks are......................... ", sel_tracks, file=stderr)
    
    # Setting whether input file has header or not
    header_sw = True
    
    if no_header:
        header_sw = False
        print("@@@Pergola_rules.py: Data file has header set to............. ", header_sw, file=stderr)

    # Handling fields to read
    if fields2read:
        print("@@@Pergola_rules.py: Fields to read from the file are......... %s" % fields2read, file=stderr)                        
    else:
        fields2read = None    
    
    # When binning data setting the window of time used in seconds
    # if not size provided set to False
    if window_size:
        print("@@@Pergola_rules.py: Window size set to....................... %d" % window_size, file=stderr)
    else:
#         window_size = 300
        window_size = False
        print("@@@Pergola_rules.py: Window size set by default to............ %d" % window_size, file=stderr)
    
    if window_mean:
        print("@@@Pergola_rules.py: Window mean set to....................... %d" % window_mean, file=stderr)
    else:      
        window_mean = False

    if value_mean:
        print("@@@Pergola_rules.py: Value mean set to....................... %d" % value_mean, file=stderr)
    else:
        value_mean = False

    if no_track_line:
        track_line=False
    else:
        track_line=True
        
    print("@@@Pergola_rules.py: track_line set to............................ %s" % track_line, file=stderr)
    
    # Handling input file field delimiter    
    if not separator:
        separator = "\t"
        print("@@@Pergola_rules.py input file field separator set by default to...... \"\\t\".", file=stderr)
    else:        
        print("@@@Pergola_rules.py input file field separator set to..... \"%s\"" % separator, file=stderr)
    
    if bed_lab_sw:
        bed_lab = True
        print("@@@Pergola_rules.py: bed_label set to......................... %s" % bed_lab, file=stderr)
    else:
        bed_lab = False

    intData = intervals.IntData(path, map_dict=map_file_dict.correspondence,
                                fields_names=fields2read,
                                header=header_sw, delimiter=separator)

    start = intData.min
    end = intData.max

    if relative_coord:
        start = 0
        end = intData.max - intData.min

    print("@@@Pergola_rules.py: min time in trajectory......................... %d" % start, file=stderr)
    print("@@@Pergola_rules.py: max time in trajectory......................... %d" % end, file=stderr)

    if min_t or min_t == 0:
        min_time = min_t
        print("@@@Pergola_rules.py: min_time set by user to.............. %d" % min_t, file=stderr)
    else:
        min_time = start

    if max_t:
        max_time = max_t
        print("@@@Pergola_rules.py: max_time set by user to............... %d" % max_t, file=stderr)
    else:
        if interval_step:
            max_time = end + interval_step
        else:
            max_time = end + 1

    # Phases option
    print("@@@Pergola_rules.py: Phases file set to............................%s" % phases, file=stderr)

    # Genome option
    print("@@@Pergola_rules.py: Genome option set to..........................%s" % genome, file=stderr)

    # Output file name
    print("@@@Pergola_rules.py: Output file/s name set t......................%s" % output_file_name, file=stderr)

    # Starting phase option
    if starting_phase:
        if phases:
            print("@@@Pergola_rules.py: Starting phase set to.............. %s" % starting_phase, file=stderr)
        else:
            raise ValueError("Starting phase needs phases option to be set to true")
    else:
        starting_phase = "light"
    if shift:
        time_shift = shift
        print("@@@Pergola_rules.py: Time shift set to....................... %d" % shift, file=stderr)
    else:
        #         window_size = 300
        time_shift = 0
        print("@@@Pergola_rules.py: Window size set by default to............ %d" % window_size, file=stderr)

    if multiply_f:
        min_time = min_time * multiply_f
        max_time = max_time * multiply_f

    if track_act: tracks2merge = parsers.read_track_actions(tracks=intData.tracks, track_action=track_act)

    data_read = intData.read(relative_coord=relative_coord,
                             intervals=intervals_gen,
                             multiply_t=multiply_f,
                             min_time=min_time, max_time=max_time,
                             int_step=interval_step)

    if genome:
        # whole trajectory
        # mapping.write_chr(data_read, min_c=start, max_c=end)
        # mapping.write_chr_sizes(data_read, min_c=start, max_c=end)

        # using min and max set by user
        mapping.write_chr(data_read)
        mapping.write_chr_sizes(data_read)

    if phases:
        # writes cytoband and light, dark and light_dark bed files
        mapping.write_cytoband(end=end, start=time_shift, start_phase=starting_phase, track_line=track_line,
                               lab_bed=bed_lab)
        # mapping.write_period_seq(start=0, end=intData.max, delta=43200, name_file="phases_dark", track_line=False)

    ## all intervals not save, in should be an option if necessary to save it
    # data_read.save_track(name_file="all_intervals")

    bed_str = data_read.convert(mode=write_format, tracks=sel_tracks,
                                tracks_merge=tracks2merge, data_types=data_types_list,
                                data_types_actions=data_types_act, window=window_size,
                                mean_win=window_mean, mean_value=value_mean, color_restrictions=d_colors_data_types)
                                #min_t_trim=min_t, max_t_trim=max_t)
    
    for key in bed_str:
        bedSingle = bed_str[key]

        output_file_n = None

        if output_file_name is not None:
            output_file_n = output_file_name + '.' + '_'.join(key)

        bedSingle.save_track(name_file=output_file_n, track_line=track_line, bed_label=bed_lab)

if __name__ == '__main__':
    exit(main())


#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
===========
Description
===========

Pergola is a python library for the conversion of behavioral data into genome browser
compatible formats. The adoption of genomic formats enables the analysis and
visualization of longitudinal behavioral data by genomic software tools.

============
Installation
============

pergola requires:

* Python >=2.6
* numpy (http://numpy.scipy.org/)
* argparse (http://code.google.com/p/argparse/)
* csv (http://www.object-craft.com.au/projects/csv/)
* biopy-isatab (https://github.com/ISA-tools/biopy-isatab)
* scipy (https://www.scipy.org/)
* pybedtools (https://daler.github.io/pybedtools/)
* pandas (https://pandas.pydata.org/)
* xlrd (https://pypi.python.org/pypi/xlrd)

Latest source code is available on GitHub::

https://github.com/cbcrg/pergola

By clicking on "Downloads", or by cloning the git repository with::

$ git clone https://github.com/cbcrg/pergola.git
    
=======
License
=======

Pergola is released under the GNU General Public License 3.0. A copy
of this license is in the LICENSE file.

"""

from pergola._version import __version__

__all__ = ['intervals', 'isatab_parser', 'mapping', 'parsers', 'tracks', 'jaaba_parsers']
# from pergola import printTest
# from intervals import IntData
# from pergola import intervals
# from tracks import DataIter

# def readme():
#     print "A library to visualize behavioral data in a genome browser"


#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

"""
unittest for pergola functions

"""
from __future__ import print_function

import unittest
from pergola import mapping
from pergola import intervals
from scripts.pergola_rules import pergola_rules
from pergola.jaaba_parsers import jaaba_scores_to_csv, jaaba_scores_to_intData
from os      import path, chdir, mkdir
from sys     import stderr
from shutil  import rmtree

# Getting the path to test files
PATH = path.abspath(path.split(path.realpath(__file__))[0])
print(path.split(path.realpath(__file__)))
PATH_DATA = path.join(path.dirname(path.dirname(path.dirname(path.abspath(__file__)))), "sample_data", '')

TEST = path.join(PATH, "uni_test", '' )

class TestTutorial(unittest.TestCase):
    """
    Testing that everything in the tutorial run smoothly        
    """ 
     
    def setUp(self):
        if not path.exists(TEST):
            mkdir(TEST)
        chdir(TEST)
        
    def test_01_correspondence_info (self):
        """
        Testing that ontology file is correctly read
        """
#         print >> stderr,"====================== test 1"
        global mappings_tutorial, exp2, exp3, exp4
        # mappings_tutorial = mapping.MappingInfo(PATH + "/feeding/f2p.txt")
        mappings_tutorial = mapping.MappingInfo(PATH_DATA + "/feeding_behavior/b2p.txt")

        msg_mappings = "Equivalences set in tutorial mapping file are not correct."
        self.assertEqual(mappings_tutorial.correspondence['EndT'], 'end', msg_mappings)
        
    def test_02_read_int_data(self):
        """
        Testing the creation of intData object using tutorial data
        """ 
#         print >> stderr,"====================== test 2"
        global data_read
        
        # Min value from tutorial file
        min = 1335985200
        max = 1337799586
        msg_int_data_min= "Min value does not correspond to tutorial files."
        msg_int_data_max= "Max value does not correspond to tutorial files."

        # int_data_tutorial = intervals.IntData(PATH + "/feeding/feeding_behavior_HF_mice.csv", map_dict=mappings_tutorial.correspondence)
        int_data_tutorial = intervals.IntData(PATH_DATA + "/feeding_behavior/feeding_behavior_HF_mice.csv", map_dict=mappings_tutorial.correspondence)

        self.assertEqual(int_data_tutorial.min, min, msg_int_data_min) 
        self.assertEqual(int_data_tutorial.max, max, msg_int_data_max)

        data_read = int_data_tutorial.read(relative_coord='False', intervals=False, multiply_t=1)

    def test_03_bed(self):
        """
        Testing the creation of bed files
        """ 
#         print >> stderr,"====================== test 3"
        write_format='bed'
         
#         data_read = int_data_tutorial.read(relative_coord='False', intervals=False, multiply_t=1)
        bed_str =  data_read.convert(mode=write_format)
          
#         for key in bed_str:
#             print key
        bedSingle_1_food_fat = bed_str[('2','food_fat')]
        bedSingle_1_food_sc = bed_str[('1','food_sc')]
        bedSingle_1_water = bed_str[('3','water')]
         
        # track containing minimum time value in order to check relative_coord   
        bedSingle_16_food_sc = bed_str[('16','food_sc')]
         
        bedSingle_1_food_fat.save_track(track_line=True, bed_label=True)                
        bedSingle_1_food_sc.save_track(track_line=True, bed_label=True)   
        bedSingle_1_water.save_track(track_line=True, bed_label=True)
         
        bedSingle_16_food_sc.save_track(track_line=True, bed_label=True)
             
    def test_04_gff(self):
        """
        Testing the creation of gff files
        """
#         print >> stderr,"====================== test 4"
        write_format='gff'

        bed_str =  data_read.convert(mode=write_format)

#         for key in bed_str:
#             print key
        bedSingle_1_food_fat = bed_str[('2','food_fat')]
        bedSingle_1_food_sc = bed_str[('1','food_sc')]
        bedSingle_1_water = bed_str[('3','water')]

        # track containing minimum time value in order to check relative_coord
        bedSingle_16_food_sc = bed_str[('16','food_sc')]

        bedSingle_1_food_fat.save_track(track_line=True, bed_label=True)
        bedSingle_1_food_sc.save_track(track_line=True, bed_label=True)
        bedSingle_1_water.save_track(track_line=True, bed_label=True)

        bedSingle_16_food_sc.save_track(track_line=True, bed_label=True)

    def test_05_bedGraph_noBinning(self):
        """
        Testing the creation of bedGraph files without window binning
        """
#         print >> stderr,"====================== test 5"
        write_format='bedGraph'

        bed_str =  data_read.convert(mode=write_format, window=False)

        bedSingle_1_food_fat = bed_str[('2','food_fat')]
        bedSingle_1_food_sc = bed_str[('1','food_sc')]
        bedSingle_1_water = bed_str[('3','water')]

        # track containing minimum time value in order to check relative_coord
        bedSingle_16_food_sc = bed_str[('16','food_sc')]

        bedSingle_1_food_fat.save_track(track_line=True)
        bedSingle_1_food_sc.save_track(track_line=True)
        bedSingle_1_water.save_track(track_line=True)

        bedSingle_16_food_sc.save_track(track_line=True)

    def test_06_bedGraph_Binning(self):
        """
        Testing the creation of bedGraph files without window binning
        """
#         print >> stderr,"====================== test 6"

        write_format='bedGraph'

        bed_str =  data_read.convert(mode=write_format, window=300)

        bedSingle_1_food_fat = bed_str[('2','food_fat')]
        bedSingle_1_food_sc = bed_str[('1','food_sc')]
        bedSingle_1_water = bed_str[('3','water')]

        # track containing minimum time value in order to check relative_coord
        bedSingle_16_food_sc = bed_str[('16','food_sc')]

        bedSingle_1_food_fat.save_track(track_line=True)
        bedSingle_1_food_sc.save_track(track_line=True)
        bedSingle_1_water.save_track(track_line=True)

        bedSingle_16_food_sc.save_track(track_line=True)

    def test_07_pergola_rules(self):
        """
        Testing pergola_rules script
        """
#         print >> stderr,"====================== test 7"

        data_in = PATH_DATA + "/feeding_behavior/feeding_behavior_HF_mice.csv"
        map_in = PATH_DATA + "/feeding_behavior/b2p.txt"
        pergola_rules(path=data_in, map_file_path=map_in, )

    def test_08_jaaba_to_pergola(self):
        """
        Testing jaaba_to_pergola script
        """
#         print >> stderr,"====================== test 8"

        data_in = PATH_DATA + "/jaaba_example/scores_chase_ctrl_pBDPGAL4.mat"
        jaaba_scores_to_csv(input_file=data_in, path_w=TEST, name_file="file_out", norm=True, data_type="chase")
        map_j = PATH_DATA + "/jaaba_example/jaaba2pergola.txt"
        int_data_j = jaaba_scores_to_intData(input_file=data_in, map_jaaba=map_j, name_file="JAABA_scores", delimiter="\t", norm=True, data_type="a")
        # print >> stderr, "Min value jaaba====== %d" % int_data_j.min

    def test_only_one_time_point(self):
        """
        Testing if files with just one coordinate for time are read correctly
        """
#         print >> stderr,"====================== test 9"

        global int_data_electro, mappings_electro
        msg_mappings = "Equivalences set in electrophysiology mapping file are not correct."

        mappings_electro = mapping.MappingInfo(PATH_DATA + "/electrophysiology/e2p.txt")        
        keys_electro = ['data_value', 'start', 'data_types', 'track']

        fields = list(mappings_electro.correspondence.values())

        self.assertEqual(keys_electro.sort(), fields.sort(), msg_mappings)

        min = 0
        max = 0.3
        msg_int_data_min= "Min value in electrophysiology data intData not correctly read."
        msg_int_data_max= "Max value in electrophysiology data intData not correctly read."

        int_data_electro = intervals.IntData(PATH_DATA + "/electrophysiology/electro_test_2f.txt", map_dict=mappings_electro.correspondence)

        self.assertEqual(int_data_electro.min, min, msg_int_data_min)
        self.assertEqual(int_data_electro.max, max, msg_int_data_max)

    def test_track_electro(self):
#         print >> stderr,"====================== test 10"

        global tracks_data_electro
        msg_track_electro = "Track electrophysiology not correctly read"

        max = 302
        tracks_data_electro = int_data_electro.read(multiply_t=1000, intervals=True)
        self.assertEqual(tracks_data_electro.max, max, msg_track_electro)

        first_item = (0.0, '-30.98', 'a', '1', 9.0)
        first_item_read = tracks_data_electro.data[0]
        self.assertEqual(tracks_data_electro.max, max, msg_track_electro)
        self.assertEqual(first_item_read, first_item, msg_track_electro)

    def test_read_int_data_from_xls(self):
        """
        Testing the creation of intData object using data in xls format
        """
        # print >> stderr,"====================== test 11"

        # Min value from tutorial file
        min = 1335985200
        max = 1337766069
        msg_int_data_min = "Min value does not correspond to tutorial files."
        msg_int_data_max = "Max value does not correspond to tutorial files."

        int_data_tutorial = intervals.IntData(PATH_DATA + "/feeding_behavior/feeding_behavior_HF_mice.xlsx",
                                              map_dict=mappings_tutorial.correspondence)

        self.assertEqual(int_data_tutorial.min, min, msg_int_data_min)
        self.assertEqual(int_data_tutorial.max, max, msg_int_data_max)

        data_read = int_data_tutorial.read(relative_coord='False', intervals=False, multiply_t=1)

    def tearDown(self):
        rmtree (TEST)
        
if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
#
#  Copyright (c) 2014-2019, Centre for Genomic Regulation (CRG).
#  Copyright (c) 2014-2019, Jose Espinosa-Carrasco and the respective authors.
#
#  This file is part of Pergola.
#
#  Pergola is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pergola is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Pergola.  If not, see <http://www.gnu.org/licenses/>.

# from distutils.core import setup
from __future__ import print_function
from setuptools import setup
from setuptools.command.install import install
from os import path, getenv
from sys import exit

PATH = path.abspath(path.split(path.realpath(__file__))[0])

# VERSION = "0.1.5"
VERSION = open("pergola/_version.py").readlines()[-1].split()[-1].strip("\"'")

TAGS = [
    "Development Status :: 2 - Pre-Alpha",
    "Environment :: Console",
    "Intended Audience :: Developers",
    "Intended Audience :: Other Audience",
    "Intended Audience :: Science/Research",
    "License :: OSI Approved :: GNU General Public License (GPL)",
    "Natural Language :: English",
    "Operating System :: MacOS",
    "Operating System :: POSIX :: Linux",
    "Operating System :: Microsoft :: Windows",
    "Programming Language :: Python",
    "Topic :: Scientific/Engineering :: Bio-Informatics",
    "Topic :: Scientific/Engineering :: Information Analysis",
    "Topic :: Scientific/Engineering :: Visualization",
    "Topic :: Software Development :: Libraries :: Python Modules",
    ]


def can_import(modname):
    """
    Test whether a module can be imported (already installed in the system)
    
    """

    try:
        __import__(modname)
    except ImportError:
        return None
    else:
        return True


def ask(string, valid_values, default=-1, case_sensitive=False):
    """ 
    Asks for a keyborad answer 
    
    """

    v = None
    if not case_sensitive:
        valid_values = [value.lower() for value in valid_values]

    while v not in valid_values:
        v = raw_input("%s [%s]" % (string,','.join(valid_values)))

        if v == '' and default >= 0:
            v = valid_values[default]
        if not case_sensitive:
            v = v.lower()

    return v


PYTHON_DEPENDENCIES = [
    ["csv"       , "Required for reading csv input files.", 1],
    ["argparse"  , "Required to read scripts arguments", 1],
    ["numpy"     , "Required for reading arrays from Jaaba matlab files and arange.", 1],
    ["bcbio"     , "Required for reading isatab files, aka biopy-isatab", 0],
    ["scipy"     , "Required for reading Jaaba matlab files.", 0],
    ["pybedtools", "Required to create pybedtools objects from Bed, BedGraph and Gff pergola objects.", 0]]


print("Checking dependencies...")
missing = False
for mname, msg, ex in PYTHON_DEPENDENCIES:

    if not can_import(mname):
        print("  *", mname, "cannot be found in your python installation.")
        print("   ->", msg)

        if ex:
            missing = True
        else:
            print('\nWARNING: It is HIGHLY RECOMMENDED to install all Pergola ' +
                  'dependencies.\nThe installation will continue. try to fix '
                  'it afterwards.')

if missing:
    exit("Essential dependencies missing, please review and install.\n")

long_description = 'A library to analyze and visualize behavioral data by unlocking genomic tools'

if path.exists ("README.rst"):
    with open('README.rst') as file:
        long_description = file.read()

class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches version in setup"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = getenv('CIRCLE_TAG')

        if tag != VERSION:
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, VERSION
            )
            exit(info)

def main():
    setup(name='pergola',
          version=VERSION,
          description='A library to analyze and visualize behavioral data by unlocking genomic tools',
          long_description= long_description,
          url='http://github.com/cbcrg/pergola',
          author='Jose Espinosa-Carrasco',
          author_email='espinosacarrascoj@gmail.com',
          license='GPLv3', #License :: OSI Approved :: GNU General Public License v3 (GPLv3)
          package_dir={'pergola': PATH + '/pergola', 'scripts': PATH + '/pergola/scripts', },
          package_data={'mypkg': ['data/sample_data/*.csv']},
          packages=['pergola', 'scripts'],
          #       scripts = ['scripts/pergola_rules.py', 'scripts/pergola_isatab.py', 'scripts/jaaba_to_pergola.py'],
          zip_safe=False,
          classifiers=TAGS,
          #       install_requires=["scipy>=0.17", "pybedtools>=0.7", "numpy>=1.10"],
          entry_points={
              'console_scripts': [
                  'pergola = scripts.pergola_rules:main',
                  'pergola_rules.py = scripts.pergola_rules:main',
                  'jaaba_to_pergola = scripts.jaaba_to_pergola:main',
                  'pergola_isatab.py = scripts.pergola_isatab:main',
              ]},
          cmdclass={
              'verify': VerifyVersionCommand,
              }
          )

if __name__ == '__main__':
    exit(main())
